import unittest 
import sys
import os

from cromulent import vocab, model
from cromulent.model import factory

class TestClassBuilder(unittest.TestCase):
	def setUp(self):
		pass

	def tearDown(self):
		pass

	def test_class(self):
		vocab.register_aat_class("TestObject1", model.ManMadeObject, "1", "example 1")
		from cromulent.vocab import TestObject1
		self.assertEqual(TestObject1._classification.id, "aat:1")

	def test_instance(self):
		vocab.register_instance("TestMaterial2", model.Material, "2", "example 2")
		self.assertTrue('TestMaterial2' in vocab.instances)
		tm2 = vocab.instances['TestMaterial2']
		self.assertEqual(tm2.id, "aat:2")

	def test_art_setter(self):
		p = model.ManMadeObject("a", art=1)
		p.label = "a"
		pj = p._toJSON()
		self.assertFalse(pj.get('classified_as', None))
		vocab.add_art_setter()
		p2 = vocab.Painting("b", art=1)
		p2j = p2._toJSON()
		# self.assertTrue("aat:300133025" in p2j['classified_as'])
		# no idea why the aat:1234 pattern doesn't work here
		# something to do with failing to set up the factory?




import codecs
import json

try:
    from collections import OrderedDict
except:
    try:
        from ordereddict import OrderedDict
    except:
        raise Exception("To run with old pythons you must: easy_install ordereddict")


fn = '../cromulent/data/crm_vocab.tsv'
fh = codecs.open(fn, 'r', 'utf-8')
lines = fh.readlines()
fh.close()

context = OrderedDict()
context['@version'] = 1.1
context['crm'] = "http://www.cidoc-crm.org/cidoc-crm/"
context['rdf'] = "http://www.w3.org/1999/02/22-rdf-syntax-ns#"
context['rdfs'] = "http://www.w3.org/2000/01/rdf-schema#"
context['dc'] = "http://purl.org/dc/elements/1.1/"
context['dcterms'] = "http://purl.org/dc/terms/"
context['schema'] = "http://schema.org/"
context['skos'] = "http://www.w3.org/2004/02/skos/core#"
context['foaf'] = 'http://xmlns.com/foaf/0.1/'
context['xsd'] = "http://www.w3.org/2001/XMLSchema#"
context["la"] = "https://linked.art/ns/terms/"
context["aat"]  = "http://vocab.getty.edu/aat/"
context["ulan"] = "http://vocab.getty.edu/ulan/"
context["tgn"] = "http://vocab.getty.edu/tgn/"

## These are only aliases. The processing is defined by the spec.
context['id'] = "@id"
context['type'] = "@type"

extension = OrderedDict()
extension['@version'] = 1.1
extension['crm'] = "http://www.cidoc-crm.org/cidoc-crm/"

for l in lines:
	l = l[:-1] # chomp
	info= l.split('\t')
	name = info[0]	
	if info[1] == "class":
		# map json key to ontology for @type:@vocab
		ctname = info[2]
		used = info[-1]
		if name.startswith("E"):
			name = "crm:%s" % name
		# split into used and other
		if used == "1":			
			context[ctname] = {"@id": name, "@context": {}}
		else:
			extension[ctname] = {"@id": name, "@context": {}}
	else:
		ctname = info[2]

		write = not ctname in ['part', 'part_of']
		# These need to be added correctly to all parents in the ontology

		dmn = info[6]
		rng = info[7]
		used = info[-2]
		mult = info[11] or '1'
		which = context if used == "1" else extension
		if which.has_key(ctname):
			print "Already found: %s   (%s vs %s)" % (ctname, context[ctname]['@id'], name)
		else:
			if rng:
				if rng[0] == "E":
					typ = "@id"
				else:
					typ = rng
			else:
				typ = "@id"

			if name.startswith("P"):
				name = "crm:%s" % name

			if write:
				if typ in ["rdfs:Literal", "xsd:dateTime", "xsd:string"]:
					which[ctname] = {"@id": name}
				elif mult == '1':
					which[ctname] = {"@id": name, "@type": typ, "@container":"@set"}
				else:
					which[ctname] = {"@id": name, "@type": typ}
			else:
				print "scoped context: %s: %s on %s" % (ctname, name, dmn)

ctxt = {"@context": context}
xctxt = {"@context": extension}

outstr = json.dumps(ctxt, indent=2)
fh = file("../cromulent/data/linked-art.json", 'w')
fh.write(outstr)
fh.close()

outstr = json.dumps(xctxt, indent=2)
fh = file("../cromulent/data/cidoc-extension.json", 'w')
fh.write(outstr)
fh.close()




import json
from cidoc_orm import factory, TimeSpan, ManMadeObject, Type, Identifier, \
	Production, Person, Place, Group, Material, Type, Mark, Right, Document, \
	Activity
import re

# Meta meta
ext_classes = {
	"TMSNumber": {"parent": Identifier, "vocab": "aat", "id": "300404621"},	
	"AccessionNumber": {"parent": Identifier, "vocab": "aat", "id": "300312355"},	
	"Inscription": {"parent": Mark, "vocab": "aat", "id": "300028702"},
	"Signature": {"parent": Mark, "vocab": "aat", "id": "300028705"},
	"Exhibition": {"parent": Activity, "vocab": "aat", "id": "300054766"},
	"Painting": {"parent": ManMadeObject, "vocab": "aat", "id": "300033618"},
	"Sculpture": {"parent": ManMadeObject, "vocab": "aat", "id": "300047090"},
	"Drawing": {"parent": ManMadeObject, "vocab": "aat", "id": "300033973"},
	"Miniature": {"parent": ManMadeObject, "vocab": "aat", "id": "300033936"},
	"Tapestry": {"parent": ManMadeObject, "vocab": "aat", "id": "300205002"},
	"Furniture": {"parent": ManMadeObject, "vocab": "aat", "id": "300037680"},
	"Mosaic": {"parent": ManMadeObject, "vocab": "aat", "id": "300015342"},
	"Photograph": {"parent": ManMadeObject, "vocab": "aat", "id": "300046300"},
	"Drawing": {"parent": ManMadeObject, "vocab": "aat", "id": "300033973"},
	"Coin": {"parent": ManMadeObject, "vocab": "aat", "id": "300037222"},
	"Vessel": {"parent": ManMadeObject, "vocab": "aat", "id": "300193015"},
	"PhotographPrint": {"parent": ManMadeObject, "vocab": "aat", "id": "300127104"},
	"PhotographAlbum": {"parent": ManMadeObject, "vocab": "aat", "id": "300026695"},
	"PhotographBook": {"parent": ManMadeObject, "vocab": "aat", "id": "300265728"}
}

# Jewelry
# Text Book Album
# Implement

# Note many sub types of Vessels, including
# Bowl, Flask, Beaker, Cup, Jar, Amphora, 

for (name,v) in ext_classes.items():
	c = type(name, (v['parent'],), {})
	c._p2_has_type = "http://vocab.getty.edu/%s/%s" % (v['vocab'], v['id'])
	globals()[name] = c

aat_type_mapping = {
	"Painting": Painting,
	"Paintings": Painting,
	"Drawing": Drawing,
	"Furniture": Furniture,
	"Coin": Coin,
	"Sculpture": Sculpture,
	"Vessels": Vessel
}

#	"panel": "300014657"  # A wooden support

aat_part_mapping = {
	"supports": "300014844"  # The thing that is painted on
}

aat_material_mapping = {
	"watercolor": "300015045",
	"oil": "300015050",
	"tempera": "300015062",
	"canvas": "300014078",
	"oak": "300012264",
	"gold leaf": "300264831",
	"paper": "300014109",
	"copper": "300011020",
	"terracotta": "300010669",
	"glass": "300010797",
	"chalk": "300011727",
	"bronze": "300010957",
	"marble": "300011443",
	"albumen silver print": "300127121",
	"gelatin silver print": "300128695",
	"silver": "300011029"
}

aat_culture_mapping = {
	"french": "300111188",
	"italian": "300111198",
	"german": "300111192",
	"dutch": "300020929"
}

dim_type_mapping = {
	"height": "300055644",
	"width": "300055647",
	"depth": "300072633",
	"diameter": "300055624",
	"weight": "300056240"
}


# Meta
class CreditLine(Right):
	def __init__(self, *args, **kw):
		super(CreditLine, self).__init__(*args, **kw)
		# XXX Find a good Type for this
		self.has_type = Type("http://example.org/ns/creditline")
CreditLine._properties['value'] = {"rdf": "rdfs:value", "range": str}

class SourceCreditLine(CreditLine):
	def __init__(self, *args, **kw):
		super(SourceCreditLine, self).__init__(*args, **kw)
		# XXX Find a good Type for this
		self.has_type = []
		self.has_type = Type("http://example.org/ns/sourcecreditline")

class Department(Group):
	def __init__(self, *args, **kw):
		super(Department, self).__init__(*args, **kw)
		self.is_current_or_former_member_of = Museum


# Monkey patch Type's _toJSON to only emit full data if not just URI+type
def typeToJSON(self, top=False):
	props = self.__dict__.keys()
	if len(props) > 3:
		return super(Type, self)._toJSON()
	else:
		return self.id

Type._toJSON = typeToJSON
Person._properties['familyName'] = {"rdf": "schema:familyName", "range": str}
Person._properties['givenName'] = {"rdf": "schema:givenName", "range": str}
ManMadeObject._properties['culture'] = {"rdf": "schema:genre", "range": Type}


factory.base_url = "http://data.getty.edu/museum/"
factory.default_lang = "en"

departments = {}
locations = {}

GettyTrust = Group("http://vocab.getty.edu/ulan/500115987")
GettyTrust.label = "J. Paul Getty Trust"
Museum = Group("http://vocab.getty.edu/ulan/500115988")
Museum.is_current_or_former_member_of = GettyTrust
Museum.label = "J Paul Getty Museum"

painting_on_re = re.compile("^(.+?) on (.+?)$")
painting_and_re = re.compile("^(.+?) and (.+?)$")

def parse_materials(materials, typ):

	mats = []
	if typ == Painting:
		# Test for X on Y
		mat = materials.lower()
		m = painting_on_re.match(mat)

		if m:
			paint = m.groups()[0]

			# x and y
			m2 = painting_and_re.match(paint)
			if m2:
				paints = m2.groups()
			else:
				paints = [paint]
			for p in paints:
				if aat_material_mapping.has_key(p):
					mats.append(Material("http://vocab.getty.edu/aat/%s" % aat_material_mapping[p]))
				else:
					pass
					# print "Paint: %s" % paint

			support = m.groups()[1]
			if aat_material_mapping.has_key(support):
				mats.append(Material("http://vocab.getty.edu/aat/%s" % aat_material_mapping[support]))
			else:
				# look for common adjectives, ()s
				swords = support.split(' ')
				for sw in swords:
					if aat_material_mapping.has_key(sw):
						mats.append(Material("http://vocab.getty.edu/aat/%s" % aat_material_mapping[sw]))					

	return mats


fh = file('record_cache.json')
data = fh.read()
fh.close()
cache = json.loads(data)


# Load up people
fh = file('500_people.json')
data = fh.read()
fh.close()
peoplel = json.loads(data)
people = {}
for who in peoplel:
	whoid = str(who['id'])
	if people.has_key(whoid):
		continue
	else:
		wrec = {'id': whoid}
		wrec['type'] = who['type']
		wrec['date'] = who['display_date']
		wrec['name'] = who['display_name']
		wrec['nationality'] = who['display_nationality']
		wrec['birthplace'] = who['display_birthplace']
		wrec['deathplace'] = who['display_deathplace']
		wrec['institution'] = who['display_institution']
		wrec['image'] = who['display_image']
		wrec['biography'] = who['display_biography']
		people[whoid] = wrec 

print "Processing..."

#recs = cache.values()
recs = [cache['645']]

ldrecs = []
for rec in recs:
	ident = str(rec['id'])

	# Build a Foo type of MMO
	clslabel = rec['classification']['name']
	clsid = str(rec['classification']['id'])
	ot = rec['object_types'] # {'primary': {}, '???': {}}
	try:
		otid = str(ot['primary']['id'])
		otlabel = ot['primary']['display_value']
	except:
		otid = ""
		otlabel = ""

	if clslabel == "Photographs":
		if otlabel == "Print":
			obj = PhotographPrint(ident)
		elif otlabel == "Album":
			obj = PhotographAlbum(ident)
		elif otlabel == "Book":
			obj = PhotographBook(ident)
		elif otlabel.lower() == "cased object":
			# Treat as print?
			obj = PhotographPrint(ident)
		else:
			print "Unknown photograph subtype: %s" % otlabel
	elif aat_type_mapping.has_key(otlabel):
		obj = aat_type_mapping[otlabel](ident)
	elif aat_type_mapping.has_key(clslabel):
		obj = aat_type_mapping[clslabel](ident)
	else:
		obj = ManMadeObject(ident)
		# print "ot: '%s' ; cls: '%s'" % (otlabel, clslabel)
		t = Type(str(clsid))
		t.label = clslabel
		obj.has_type = t

	tms = TMSNumber(ident)
	tms.value = ident
	obj.is_identified_by = tms

	recno = rec['number']
	accno = AccessionNumber(recno)
	accno.value = recno
	obj.is_identified_by = accno

	obj.label = rec['title']
	try:
		obj.description = rec['description']['display']['value']
	except:
		pass

	production = Production(ident)
	obj.was_produced_by = production
	ts = TimeSpan(ident)
	ts.description = rec['date']
	# XXX Parse date string for dates
	production.has_timespan = ts

	# XXX if there are multiple makers with different roles,
	# create a super Production with components, and each
	# role gets a separate component

	for mk in rec['makers']:
		mkid = str(mk['id'])
		role = mk['role']

		who = Person(mkid)

		# Find in person db or deref
		first = mk['name_first']
		last = mk['name_last']

		try:
			person = people[mkid]
			who.label = person['name']
			who.description = person['biography']
			who.givenName = first
			who.familyName = last
			who.birthPlace = Place()
			who.deathPlace = Place()
			who.birthDate = ""
			who.deathDate = ""
		except:
			pass

		production.carried_out_by = who
		# XXX Link to ULAN

	# found, depicted, created

	if rec['places'] and rec['places'].has_key('place_created'):
		p = rec['places']['place_created']
		pid = str(p['id'])
		where = Place(pid)
		where.label = p['display_value']
		production.took_place_at = where

	# XXX Check for place_depicted (find out all possible keys)

	m = Material(ident)
	m.description = rec['medium']
	obj.consists_of = m
	mats = parse_materials(rec['medium'], obj.__class__)
	if mats:
		for mat in mats:
			m.defines_typical_wholes_for = mat

	dpt = rec['department']
	dptid = dpt['id']
	try:
		dept = departments[dptid]
	except:
		dept = Department(str(dpt['id']))
		dept.label = dpt['name']
		departments[dptid] = dept
	obj.has_current_owner = dept

	if rec['location']:
		loc = rec['location'][0]
		locid = str(loc['id'])
		try:
			where = locations[locid]
		except:
			where = Place(locid)
			where.label = loc['name']
			locations[locid] = where
		obj.has_current_location = where

	culture = Type("culture")
	culture.label = rec['culture']
	obj.culture = culture
	# XXX Map to AAT

	if rec['markings']:
		markings = Mark()
		markings.description = rec['markings']
		obj.shows_visual_item = markings
	if rec['signature']:
		sig = Signature()
		sig.description = rec['signature']
		obj.shows_visual_item = sig
	if rec['inscription']:
		insc = Inscription()
		insc.description = rec['inscription']
		obj.shows_visual_item = insc

	if rec.has_key('creditline'):
		credit = CreditLine()
		credit.value = rec['creditline']
		obj.is_subject_to = credit
	if rec['source_creditline']:
		srcCredit = SourceCreditLine()
		srcCredit.value = rec['source_creditline']
		obj.is_subject_to = srcCredit

	if rec['bibliography']:
		bx = 0
		for bib in rec['bibliography']:
			bt = bib['display_source_type']
			bv = bib['display_value']
			doc = Document("%s/%s" % (ident, str(bx)))
			bx += 1
			doc.label = bv
			doc.has_type = Type(bt)
			# XXX extract actual bib data and map to something sensible
			obj.is_documented_in = doc

	if rec['provenance']:
		for prov in rec['provenance']:
			date = prov['display_date']
			pid = str(prov['id'])
			who = prov['display_constituent']
			# XXX Parse constituent and map to provenance patterns


	if rec['related_exhibitions']:
		for exh in rec['related_exhibitions']:
			exhid = str(exh['record_identifier'])
			ttl = exh['display_title']
			dates = exh['display_dates']

			exhibition = Exhibition(exhid)
			exhibition.label = ttl
			if dates:
				ts = TimeSpan(exhid)
				ts.description = dates
				exhibition.has_timespan = ts
				# XXX parse for begin, end dates

			vens  = exh['display_venues']
			for v in vens:
				name = v['display_name']
				loc = v['display_location']
				vid = str(v['record_identifier'])
				vdates = v['display_dates']

				venue = Activity(vid)
				venue.label = name
				if vdates:
					vts = TimeSpan(vid)
					vts.description = vdates
					venue.has_timespan = vts
					# XXX Parse for begin, end dates
				if loc:
					place = Place(vid)
					place.description = loc
					# XXX Parse location
					venue.took_place_at = place
				exhibition.consists_of = venue

				# XXX Catalog Number is a Document that documents the Venue or Exhibition

			obj.was_present_at = exhibition

	ldrecs.append(obj)
	# print factory.toString(obj, compact=False)
	#break






import unittest 

try:
	from collections import OrderedDict
except:
	# 2.6
	from ordereddict import OrderedDict

from cromulent import reader
from cromulent.model import factory, Person, DataError, BaseResource, Dimension

class TestReader(unittest.TestCase):

	def setUp(self):
		self.reader = reader.Reader()
		# ensure we can use parent_of
		Person._properties['parent_of']['okayToUse'] = 1

	def test_read(self):
		self.assertRaises(DataError, self.reader.read, "")
		self.assertRaises(DataError, self.reader.read, "This is not JSON")
		self.assertRaises(DataError, self.reader.read, "{}")

		whostr = '{"type": "Person", "label": "me"}'
		self.assertTrue(isinstance(self.reader.read(whostr), Person))

		whostr = '{"@context": "fishbat", "type": "Person", "label": "me"}'
		self.assertTrue(isinstance(self.reader.read(whostr), Person))

		levelstr = '{"type": "Person", "parent_of": {"type": "Person", "label": "child"}}'
		self.assertTrue(isinstance(self.reader.read(levelstr).parent_of, Person))

		basestr = '{"label": "base"}'
		self.assertTrue(isinstance(self.reader.read(basestr), BaseResource))

		unknown = '{"type":"FishBat"}'
		self.assertRaises(DataError, self.reader.read, unknown)

		unknown2 = '{"type":"Person", "fishbat": "bob"}'
		self.assertRaises(DataError, self.reader.read, unknown)

		# somewhere else, rdf_value has been added
		try:
			del Dimension._properties['value']
		except:
			# maybe not?
			pass




from lxml import etree
import codecs
import json

default_key_order = 10000

NS = {'rdf':"http://www.w3.org/1999/02/22-rdf-syntax-ns#",
	'xsd':"http://www.w3.org/2001/XMLSchema#",
	'rdfs':"http://www.w3.org/2000/01/rdf-schema#",
	'dcterms':"http://purl.org/dc/terms/",
	'owl':"http://www.w3.org/2002/07/owl#",
	'crm':"http://www.cidoc-crm.org/cidoc-crm/",
	'skos':"http://www.w3.org/2004/02/skos/core#",
	'xml': "http://www.w3.org/XML/1998/namespace"
}

fh = file('bibframe.rdf')
data = fh.read()
fh.close()
dom = etree.XML(data)
stuff = []

property_overrides = {}

classes = dom.xpath("//rdfs:Class", namespaces=NS)

if not classes:
	classes = dom.xpath('//owl:Class', namespaces=NS)

for c in classes:
	label = c.xpath('./rdfs:label/text()', namespaces=NS)[0]
	try:
		comment = c.xpath('./rdfs:comment/text()', namespaces=NS)
		if not comment:
			comment = c.xpath('./skos:definition/text()', namespaces=NS)
		if comment:
			comment = comment[0]
		comment = comment.strip()
		comment = comment.replace('\n', '\\n').replace('\t', ' ')
	except:
		comment = ""
	name = c.xpath('@rdf:about', namespaces=NS)[0]

	subCls = c.xpath('./rdfs:subClassOf/@rdf:resource', namespaces=NS)
	if subCls:
		# could be multiples
		subCls = '|'.join(subCls)
	else:
		subCls = ""

	uc1 = name.rfind("/")
	ccname = name[uc1+1:]
	ccname = ccname.replace("_or_", "_Or_").replace("_of_", "_Of_")
	ccname = ccname.replace('-', '').replace('_', '')

	stuff.append([name, "class", ccname, label, comment, subCls])

props = dom.xpath("//rdf:Property",namespaces=NS)
if not props:
	props = dom.xpath('//owl:DatatypeProperty', namespaces=NS)
	props.extend(dom.xpath('owl:ObjectProperty', namespaces=NS))

for p in props:
	label = p.xpath('./rdfs:label/text()', namespaces=NS)[0]
	try:
		comment = p.xpath('./rdfs:comment/text()', namespaces=NS)
		if not comment:
			comment = c.xpath('./skos:definition/text()', namespaces=NS)
		if comment:
			comment = comment[0]
		comment = comment.strip()
		comment = comment.replace('\n', '\\n').replace('\t', ' ')
	except:
		comment = ""

	name = p.xpath('@rdf:about', namespaces=NS)[0]
	domn = p.xpath('./rdfs:domain/@rdf:resource', namespaces=NS)
	if domn:		
		domn = domn[0]
		for (k,v) in NS.items():
			domn = domn.replace(v,"%s:" % k)
	else:
		domn = ""
	rang = p.xpath('./rdfs:range/@rdf:resource', namespaces=NS)
	if rang:
		rang = rang[0]
		for (k,v) in NS.items():
			rang = rang.replace(v,"%s:" % k)
	else:
		rang = ""
	subProp = p.xpath('./rdfs:subPropertyOf/@rdf:resource', namespaces=NS)
	if subProp:
		subProp = subProp[0]
	else:
		subProp = ""

	inverse = p.xpath('./owl:inverseOf/@rdf:resource', namespaces=NS)
	if inverse:
		inverse = inverse[0]
	else:
		inverse = ""

	uc1 = name.find("_")
	pno = name[:uc1]
	if property_overrides.has_key(pno):
		ccname = property_overrides[pno]
	else:
		ccname = name[uc1+1:]
		ccname = ccname.replace("-", "")
		if ccname.startswith("is_"):
			ccname = ccname[3:]
		elif ccname.startswith("has_") or ccname.startswith("had_") or ccname.startswith("was_"):
			ccname = ccname[4:]

	# koi = str(key_order_hash.get(ccname, default_key_order))
	koi = "10000"
	stuff.append([name, "property", ccname, label, comment, subProp, domn, rang, inverse, koi])

outdata = '\n'.join(['\t'.join(x) for x in stuff])
fh = codecs.open('bibframe_vocab.tsv', 'w', 'utf-8')
fh.write(outdata)
fh.close()




import unittest 

try:
	from collections import OrderedDict
except:
	# 2.6
	from ordereddict import OrderedDict

from cromulent import multiple_instantiation as mi
from cromulent.model import factory, Person, DataError, Dimension


class TestMIClasses(unittest.TestCase):

	def test_destruction(self):
		expect = OrderedDict([('id', u'http://lod.example.org/museum/Activity/1'), 
			('type', ['Destruction', 'Activity']), ('label', "Test Destruction")])
		mi.DestructionActivity._okayToUse = 1
		da = mi.DestructionActivity("1")
		da.label = "Test Destruction"
		factory.context_uri = ""
		dajs = factory.toJSON(da)
		self.assertEqual(dajs, expect)




from lxml import etree
import codecs

fh = file('cidoc.xml')
data = fh.read()
fh.close()

NS = {'rdf':"http://www.w3.org/1999/02/22-rdf-syntax-ns#",
	'xsd':"http://www.w3.org/2001/XMLSchema#",
	'rdfs':"http://www.w3.org/2000/01/rdf-schema#",
	'dcterms':"http://purl.org/dc/terms/",
	'owl':"http://www.w3.org/2002/07/owl#",
	'crm':"http://www.cidoc-crm.org/cidoc-crm/",
	'xml': "http://www.w3.org/XML/1998/namespace"
}

dom = etree.XML(data)
names = []
inverses = {}

props = dom.xpath("//rdf:Property",namespaces=NS)
for p in props:
	name = p.xpath('@rdf:about', namespaces=NS)[0]
	names.append(name)

for p in props:
	name = p.xpath('@rdf:about', namespaces=NS)[0]	
	fu = name.find('_')
	pid = name[:fu]
	if pid[-1] in ['a', 'b']:
		# No inverses for botb eote
		continue
	inverse = ""
	if pid[-1] == "i":
		pid = pid[:-1]
	else:
		pid = pid + "i"
	pid += "_"

	for i in names:
		if i.startswith(pid) and i != name:
			inverse = i
			break
	if inverse:
		inverses[name] = inverse

# Now print ONLY the inverses
outlines = [
'<rdf:RDF xml:lang="en" xmlns:owl="http://www.w3.org/2002/07/owl#" xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#" xmlns:rdfs="http://www.w3.org/2000/01/rdf-schema#" xml:base="http://www.cidoc-crm.org/cidoc-crm/" xmlns:la="http://linked.art/ns/terms/">'
]

for n in names:
	if n in inverses:
		outlines.append('  <rdf:Property rdf:about="%s">' % n )
		outlines.append('    <owl:inverseOf rdf:resource="%s"/>' % inverses[n])
		outlines.append('  </rdf:Property>')
outlines.append('</rdf:RDF>')
outstr = '\n'.join(outlines)

fh = file('data/inverses.xml', 'w')
fh.write(outstr)
fh.close()




from lxml import etree
import json
import csv
import codecs
import re
import os
import sys
from dateutil.parser import parse as dateparse

# for cidoc_orm, see: https://github.com/azaroth42/Python-CIDOC-ORM
from cidoc_orm import factory, TimeSpan, Identifier, LegalBody, \
	Production, Actor, Place, Group, Material, Mark, \
	Activity, InformationObject, Purchase, Acquisition, MonetaryAmount, \
	Currency, MeasurementUnit, Dimension, PhysicalObject, VisualItem, Title

from aat_mapping import ManMadeObject, Type, Person, materialTypes, register_aat_class, \
	Painting, Sculpture, Drawing, Miniature, Graphic, Enamel, Tapestry, Mosaic, \
	Embroidery, Furniture, LocalNumber, dimensionUnits


PhysicalObject._properties['had_starting_price'] = {"rdf": "gri:had_starting_price", "range": MonetaryAmount}

cmUnit = dimensionUnits['cm']

# Cache of repeated Objects
catalogO = {}
placeO = {}
nationalityO = {}
personO = {}

materialO = {}

bad_price = {}
bad_dates = {}
bad_types = {}
bad_materials = {}

# XXX -- Distinguish Local from Lugt
register_aat_class("LugtNumber", Identifier, "300404621")

factory.base_url = "http://data.getty.edu/provenance/"
factory.default_lang = "en"

objTypeMap = {
	u'gem\xe4lde': Painting,
	'skulptur': Sculpture,
	'zeichnung': Drawing,
	'miniatur': Miniature,
	'graphik': Graphic,
	'painting': Painting,
	'enamel': Enamel,
	'miniature': Miniature,
	'sculpture': Sculpture,
	'drawing': Drawing,
	'tapestry': Tapestry,
	'embroidery': Embroidery,
	'furniture': Furniture,
	'mosaic': Mosaic,
	'watercolor': Painting 
}


r = "(je|l|h|d|b|durchm|durchmesser|dm[.]?)?[ ]*(ca.|h)?[ ]*([0-9,.]+)([ ]*(cm)?[ ]*x[ ]*([0-9,.]+)[ ]*(cm)?)?"
dimre = re.compile(r)
# dim1 = groups()[2], dim2 = groups()[5]


def process_record(rec):
	recData = {}
	for elm in rec.getchildren():
		tag = elm.tag
		curr = recData.get(tag, None)

		if elm.getchildren():
			value = elm
		else:
			value = elm.text

		if curr is None:
			recData[tag] = value
		elif type(recData[tag]) == list:
			recData[tag].append(value)
		else:
			recData[tag] = [curr, value]

	cno = recData['Catalogue_No']
	try:
		catalog = catalogO[cno]
		auction = catalog.refers_to

		# Try and update end of timespan
		if not hasattr(auction.has_timespan, 'end_of_the_end'):
			sed = recData.get('Sale_End_Date', '')			
			if sed:
				try:
					dt = dateparse(sed)
					span.end_of_the_end = "%s-%s-%sT00:00:00 CET" % (dt.year, dt.month, dt.day)
				except:
					try:
						bad_dates[sed] += 1
					except:
						bad_dates[sed] = 1

	except:
		catalog = InformationObject(cno)
		catalogO[cno] = catalog
		auction = Activity(cno)
		catalog.refers_to = auction
		catalog.has_representation = VisualItem(recData['GSC_link_to_pdf'])
		catalog.is_identified_by = LocalNumber(cno)

		# Auction date
		span = TimeSpan(cno)
		sbd = recData['Sale_Begin_Date']
		sed = recData.get('Sale_End_Date', '')
		try:
			dt = dateparse(sbd)
			span.begin_of_the_begin = "%s-%s-%sT00:00:00 CET" % (dt.year, dt.month, dt.day)
		except:
			try:
				bad_dates[sbd] += 1
			except:
				bad_dates[sbd] = 1
		if sed:
			try:
				dt = dateparse(sed)
				span.end_of_the_end = "%s-%s-%sT00:00:00 CET" % (dt.year, dt.month, dt.day)
			except:
				try:
					bad_dates[sed] += 1
				except:
					bad_dates[sed] = 1
		auction.has_timespan = span

		# Auction location
		city = recData['City_of_Sale']
		try:
			cityPlace = placeO[city]
		except:
			cityPlace = Place(city)
			cityPlace.label = city
			placeO[city] = cityPlace
		auction.took_place_at = cityPlace

		try:
			country = cityPlace.falls_within
		except:
			try:
				country = recData['Country_Auth']
				countryPlace = Place(country)
				countryPlace.label = country
				cityPlace.falls_within = countryPlace
			except:
				# No country
				pass

		try:
			# Auction House
			house = recData['Auction_House']
			try:
				ahouse = houseO[house]
			except:
				ahouse = LegalBody(house)
				ahouse.label = house
			auction.carried_out_by = ahouse
		except:
			# No auction house? :(
			pass

	try:
		lno = recData["Lot_Number"]
	except:
		print "No lot number for %s" % recData['recno']
		return

	cnolot = cno +'/'+ lno

	# Build an aggregation of objects for the lot
	lotset = PhysicalObject(cnolot + "-set")
	# InfoObj for the entry
	entry = InformationObject(cnolot)
	entry.refers_to = lotset
	catalog.is_composed_of = entry

	if recData.has_key('Price'):
		pr = recData['Price']
		# Process prinfo

		if type(pr) == list:
			pr = pr[0]
		if not type(pr) in [str, unicode]:
			try:	
				pr = pr.text
			except:
				pr = ""

		pr = pr.replace("[?]", "")
		pr = pr.replace('?', '')
		pr = pr.strip()

		if pr:
			pr = pr.replace('1/2', '.5')
			pr = pr.replace('1/4', '.25')
			pr = pr.replace('3/4', '.75')
			pr = pr.replace(' .', '.')

			# Unknown:  x-y-z  x.y.z
			# x"y' x=y  x:y x=y-z
			# 

			if pr.find(" frs") > -1:
				curr = Currency("francs")
				curr.label = "francs"
				pr = pr.replace(' frs', '')
			elif pr.find(" fl") > -1:
				curr = Currency("fl.s")
				curr.label = "fl.s"
				pr = pr.replace(' fl', '')
			elif pr.find(" livres") > -1:
				curr = Currency("pounds")
				curr.label = "pounds"
				pr = pr.replace(' livres', '')
			else:
				curr = None

			pr = pr.replace('[or]', 'or')
			oidx = pr.find(' or ') 
			if oidx > -1:
				pr = pr[:oidx]
				pr = pr.strip()

			fidx = pr.find(' for ')
			if fidx > -1:
				pr = pr[:fidx]
				pr = pr.strip()

			try:
				p = float(pr)				
			except:
				p = -1
				try:
					bad_price[pr] += 1
				except:
					bad_price[pr] = 1
			if p >= 0:
				amnt = MonetaryAmount(cnolot + "-start")
				amnt.has_value = p
				if curr:
					amnt.has_currency = curr
		 		lotset.had_starting_price = amnt


	# Build the object

	try:
		typs = recData['Object_Types'].xpath('./Object_Type/text()')
		ot = typs[0]
		cls = objTypeMap[ot]
	except:
		cls = ManMadeObject
		try:
			bad_types[ot] += 1
		except:
			bad_types[ot] = 1

	obj = cls(cnolot)
	lotset.is_composed_of = obj

	title = Title(cnolot)
	obj.has_title = title
	try:
		title.value = recData['Title']
	except:
		title.value = "[No Title Known]"
	if recData.has_key('Title_Modifier'):
		title.has_note = unicode(recData['Title_Modifier'])

	if recData.has_key('Materials'):
		for mat in recData['Materials'].xpath('./Material/text()'):
			ot = mat.lower()
			ot = ot.replace(',', '')
			ot = ot.replace('#', '')
			ot = ot.replace('.', '')
			ot = ot.replace('?', '')
			ot = ot.replace('auf', '')
			ot = ot.replace('und', '')
			ot = ot.replace("on", " ")
			ot = ot.replace("and", " ")			
			ot = ot.replace("  ", ' ')

			words = ot.split(' ')
			obj.consists_of = []
			for w in words:
				if w:
					try:
						material = materialO[w]
					except:
						material = Material(w)
						materialO[w] = material
						material.value = w
					obj.consists_of = material

	if recData.has_key("Dimensions"):
		for dimtext in recData['Dimensions'].xpath('./Dimension_Text/text()'):
			m = dimre.match(dimtext)
			if m:
				d1 = m.groups()[2]
				d2 = m.groups()[5]

				dim1 = Dimension(cnolot + "_d1")
				dim1.has_value = d1
				dim1.has_unit = cmUnit
				obj.has_dimension = dim1

				if d2:
					dim2 = Dimension(cnolot + "_d2")
					dim2.has_value = d2				
					dim2.has_unit = cmUnit
					obj.has_dimension = dim2

			else:
				#print "Can't handle dimension data:"
				#print dimtext			
				try:
					bad_materials[dimtext] += 1
				except:
					bad_materials[dimtext] = 1

	# Artist could be modified by Attrib_Mod
	# e.g. zugeschrieben --> attributed (2200)
	# Kopie von  --> copy from (1)
	# stil --> style [of] (24)

	# Artist
	if recData.has_key('Artist'):
		arts = recData['Artist']
		if type(arts) != list:
			arts = [arts]
		for artist in arts:
			va = artist.xpath('./Verb_Artist/text()')
			aa = artist.xpath('./Artist_Auth/text()')
			natl = artist.xpath('./Nationality/text()')
			mod = artist.xpath('./Attrib_Mod/text()')

			# first try to detect non names
			if aa:
				aa = unicode(aa[0])
				if aa == "NEW":
					# treat as if not present
					pass
				elif aa.startswith('['):
					# anonymous artist with some known features
					pass
				aname = aa.lower()
				aname = aname.replace(" ", "_")
				try:
					who = personO[aname]
				except:
					who = Person(aname)
					personO[aname] = who
					# put verbatim name somewhere
					# and authority name in p131 is identified by
					who.label = aa

					if natl: 
						natl = unicode(natl[0])
						try:
							nat = nationalityO[natl]
						except:
							nat = Group(natl)
							nat.label = natl
						who.is_current_or_former_member_of = nat

				# one production event per artist?
				prod = Production(cnolot + aname)
				prod.carried_out_by = who
				obj.was_produced_by = prod 

	# seller if we know
	seller = None
	if recData.has_key("Seller"):
		sells = recData['Seller']
		if type(sells) != list:
			sells = [sells]
		sx = 0
		for s in sells:
			try:
				lbl = unicode(s.xpath("./Seller_Auth/text()")[0])
			except:	
				try:
					lbl = unicode(s.xpath("./Verb_Seller/text()")[0])				
				except:
					# ???!!!
					continue
			end = "-seller"
			if sx:
				end += "-%s" % sx
			seller = Actor(cnolot + end)
			seller.label = lbl
			obj.has_former_or_current_owner = seller
			sx += 1

	try:
		txn = recData['Transaction']
		txn = txn.lower()
	except:
		txn = "unknown"
	txn = txn.replace('[?]', '')
	txn = txn.replace("unknown or ", "")
	txn = txn.replace(" or unknown", "")
	txn = txn.strip()

	if txn.find(" or ") > -1:
		# Don't know what to do with X or Y
		print "Not processing or for txn"
		return

	if txn in ["sold", "bought in", "passed"]:
		lot = Activity(cnolot)
		auction.consists_of = lot
		lot.used_specific_object = lotset
		
		span = TimeSpan(cnolot)
		try:
			date = recData['Lot_Sale_Date']
			# NB: this is going to end up strange due to UK vs EU timezones
			try:
				dt = dateparse(date)
				begin = "%s-%s-%sT00:00:00 CET" % (dt.year, dt.month, dt.day)
				end = "%s-%s-%sT23:59:59 CET" % (dt.year, dt.month, dt.day)
				span.begin_of_the_begin = begin
				span.end_of_the_end = end
				lot.has_timespan = span
			except:
				try:
					bad_dates[date] += 1
				except:
					bad_dates[date] = 1
		except:
			print "No Sale Date: %s" % recData['recno']

		ln = recData.get('Lot_Notes', None)
		if ln is not None:
			lot.has_note = unicode(ln)

		entry.refers_to = lot

	if txn in ['sold', 'bought in']:
		# Bidding activity exists
		bidding = Activity(cnolot + "-bidding")
		lot.consists_of = bidding

	if txn == "sold":
		txn = Purchase(cnolot+"-transaction")
		lot.consists_of = txn
		bidding.occurs_before = txn

		acq = Purchase(cnolot + "-acquisition")
		txn.consists_of = acq
		acq.transferred_title_of = obj

		if seller:
			sellers = obj.has_former_or_current_owner
			if type(sellers) == list:
				for s in sellers:
					acq.transferred_title_from = s
			else:	
				acq.transferred_title_from = sellers			

		bx = 0
		if recData.has_key("Buyer"):
			buys = recData['Buyer']
			if type(buys) != list:
				buys = [buys]
			for b in buys:
				try:
					lbl = unicode(b.xpath("./Buyer_Auth/text()")[0])
				except:
					try:
						lbl = unicode(b.xpath("./Verb_Buyer/text()")[0])					
					except:
						# ???!!!
						continue

				end = "-buyer"
				if bx:
					end += "-%s" % bx
				buyer = Actor(cnolot + end)
				buyer.label = lbl
				acq.transferred_title_to = buyer
				bx += 1


	return catalog



recdir = '/Users/rsanderson/Box Sync/PI_Sales/'
files = ['salesdb1.xml', 'salesdb2.xml', 'salesdb3.xml', 'salesdb4.xml']
files = ['salesdb1.xml']

for fn in files:
	rec = []
	fh = file(os.path.join(recdir, fn))
	#fh = codecs.open(os.path.join(recdir, fn), 'r', 'iso-8859-1')

	# chomp first three lines
	fh.readline() ; fh.readline() ; fh.readline()

	header = '<?xml version="1.0" encoding="UTF-8"?>\n'

	line = fh.readline()
	while line != "</root>\n":
		while (line.find('</record>') == -1):
			rec.append(line)
			line = fh.readline()
			if not line:
				break
		rec.append(line)	

		data = ''.join(rec)
		data = header + data
		data = data.replace("Catalogue_No.", "Catalogue_No")
		data = data.replace("Country_Auth.", "Country_Auth")
		data = data.replace('\x04', '')
		data = data.replace('\x1f', '')

		try:
			dom = etree.XML(data)
		except:
			print "Invalid record data: %s" % data[:200]

		top = process_record(dom)
		# break

		line = fh.readline()
		rec = []

	fh.close()




import unittest 
import sys
import os
import shutil
import json

try:
	from collections import OrderedDict
except:
	# 2.6
	from ordereddict import OrderedDict

from cromulent import model

class TestFactorySetupDefaults(unittest.TestCase):

	def test_init_defaults(self):
		self.assertEqual(model.factory.base_url, 'http://lod.example.org/museum/')
		self.assertEqual(model.factory.debug_level, 'warn')
		self.assertEqual(model.factory.log_stream, sys.stderr)
		self.assertFalse(model.factory.materialize_inverses)
		self.assertFalse(model.factory.full_names)
		# Don't test orders, as these will change repeatedly

class TestFactorySetup(unittest.TestCase):

	def setUp(self):
		model.factory.base_url = 'http://data.getty.edu/provenance/'
		model.factory.base_dir = 'tests/provenance_base_dir'
		model.factory.default_lang = 'en'
		#model.factory.context_uri = 'http://www.cidoc-crm.org/cidoc-crm/'

	def tearDown(self):
		model.factory.base_url = 'http://lod.example.org/museum/'
		model.factory.log_stream = sys.stderr
		model.factory.debug_level = 'warn'

	def test_base_url(self):
		self.assertEqual(model.factory.base_url, 'http://data.getty.edu/provenance/')

	def test_base_dir(self):
		self.assertEqual(model.factory.base_dir, 'tests/provenance_base_dir')

	def test_default_lang(self):
		self.assertEqual(model.factory.default_lang, 'en')

	#def test_context_uri(self):
	#	self.assertEqual(model.factory.context_uri, 'http://www.cidoc-crm.org/cidoc-crm/')

	def test_set_debug_stream(self):
		strm = open('err_output', 'w')
		model.factory.set_debug_stream(strm)
		self.assertEqual(model.factory.log_stream, strm)

	def test_set_debug(self):
		model.factory.set_debug('error_on_warning')
		self.assertEqual(model.factory.debug_level, 'error_on_warning')
		self.assertRaises(model.ConfigurationError, model.factory.set_debug, 'xxx')
		self.assertRaises(model.MetadataError, model.factory.maybe_warn, "test")

	def test_load_context(self):
		self.assertRaises(model.ConfigurationError, model.factory.load_context, 
			"foo", {"foo":"does_not_exist.txt"})
		model.factory.load_context("foo", {"foo":"tests/test_context.json"})
		self.assertEqual(model.factory.context_json, {"@context":{"id":"@id"}})
		self.assertRaises(model.ConfigurationError, model.factory.load_context, "", {})

class TestFactorySerialization(unittest.TestCase):

	def setUp(self):
		self.collection = model.InformationObject('collection')
		self.collection.label = "Test Object"

	def test_broken_unicode(self):
		model.factory.debug_level = "error_on_warning"
		try:
			badval = b"\xFF\xFE\x02"
		except:
			badval = "\xFF\xFE\x02"
		badjs = {"label": badval}
		self.assertRaises(model.MetadataError, model.factory._buildString,
			js=badjs)

	def test_toJSON(self):
		model.factory.context_uri = 'http://lod.getty.edu/context.json'
		expect = OrderedDict([
			('@context', model.factory.context_uri),
			('id', u'http://lod.example.org/museum/InformationObject/collection'), 
			('type', 'InformationObject'), ('label', 'Test Object')])
		outj = model.factory.toJSON(self.collection)
		self.assertEqual(expect, outj)

	def test_toJSON_full(self):
		expect = OrderedDict([(u'@context', 'http://lod.getty.edu/context.json'), 
			(u'@id', u'http://lod.example.org/museum/Person/1'), (u'@type', u'crm:E21_Person'),
			('rdfs:label', 'Test Person')])
		model.factory.context_uri = 'http://lod.getty.edu/context.json'
		model.factory.full_names = True
		p = model.Person("1")
		p.label = "Test Person"
		outj = model.factory.toJSON(p)
		self.assertEqual(expect, outj)
		# reset
		model.factory.full_names = False
		model.factory.context_uri = ""

	def test_only_type(self):	
		what = model.InformationObject('collection')		
		expect = '"http://lod.example.org/museum/InformationObject/collection"'
		outs = model.factory.toString(what)
		self.assertEqual(expect, outs)

	def test_toString(self):
		expect = u'{"id":"http://lod.example.org/museum/InformationObject/collection","type":"InformationObject","label":"Test Object"}'
		outs = model.factory.toString(self.collection)
		self.assertEqual(expect, outs)

	def test_toFile(self):
		self.assertRaises(model.ConfigurationError, model.factory.toFile, self.collection)
		# Test auto filename determination
		model.factory.base_dir = 'tests'
		model.factory.toFile(self.collection)
		self.assertTrue(os.path.isfile('tests/InformationObject/collection.json'))
		# Test explicit filename setting
		model.factory.toFile(self.collection, filename='tests/fishbat.bar')
		self.assertTrue(os.path.isfile('tests/fishbat.bar'))
		# Tidy up
		shutil.rmtree('tests/InformationObject')

	def test_breadth(self):
		x = model.TransferOfCustody()
		e = model.Activity()
		fr = model.Group()
		to = model.Group()
		w = model.ManMadeObject()
		fr.label = "From"
		to.label = "To"
		x.transferred_custody_of = w
		x.transferred_custody_from = fr
		x.transferred_custody_to = to
		e.used_specific_object = w
		e.carried_out_by = to
		w.current_owner = fr
		x.specific_purpose = e
		js = model.factory.toJSON(x)
		# Okay ... if we're breadth first, then custody_from is a resource
		# And now it's the first in the list
		self.assertTrue(isinstance(js['transferred_custody_from'][0], OrderedDict))

	def test_string_list(self):
		x = model.Activity()
		x.label = ["Label 1", "Label 2"]
		js = model.factory.toJSON(x)
		self.assertTrue(js['label'] == x.label)

	def test_external(self):
		x = model.ExternalResource(ident="1")
		model.factory.elasticsearch_compatible = 1
		js = x._toJSON()
		self.assertTrue(type(js) == dict)
		model.factory.elasticsearch_compatible = 0
		js = x._toJSON()
		# testing unicode in 2, str in 3 :(
		self.assertTrue(type(js) != dict)		

class TestProcessTSV(unittest.TestCase):

	def test_process_tsv(self):
		expect = {u'subs': [u'E84_Information_Carrier'], u'label': u'Man-Made Object', u'className': u'ManMadeObject', 
		u'subOf': u'E19_Physical_Object|E24_Physical_Man-Made_Thing', u'props': [], u'class': None, u'okay': u'1'}
		fn = 'cromulent/data/crm_vocab.tsv'
		vocabData = model.process_tsv(fn)
		man_made = vocabData['E22_Man-Made_Object']
		del man_made['desc']  # too long and volatile
		try:
			self.assertEqual(expect, man_made)
		except:
			expect[u'subs'] = [u'ore:Proxy', u'E84_Information_Carrier']
			self.assertEqual(expect, man_made)

class TestBuildClasses(unittest.TestCase):

	def test_build_classes(self):
		tsv = "ClassName_full\tclass\tClassName_py\tClass Label\tClass Description\t\t1\t\n"
		fh = open('tests/temp.tsv', 'w')
		fh.write(tsv)
		fh.close()
		model.build_classes("tests/temp.tsv", "ClassName_full")
		from cromulent.model import ClassName_py
		self.assertEqual('Class Description', ClassName_py.__doc__)
		os.remove('tests/temp.tsv')

class TestBuildClass(unittest.TestCase):

	def test_build_class(self):
		tsv = "ClassName_full\tclass\tClassName_py2\tClass Label\tClass Description\t\t1\t\n"
		fh = open('tests/temp.tsv', 'w')
		fh.write(tsv)
		fh.close()
		vocabData = model.process_tsv('tests/temp.tsv')
		model.build_class('ClassName_full', model.BaseResource, vocabData)
		from cromulent.model import ClassName_py2
		self.assertEqual('Class Description', ClassName_py2.__doc__)
		os.remove('tests/temp.tsv')

class TestAutoIdentifiers(unittest.TestCase):

	def test_bad_autoid(self):
		model.factory.auto_id_type = "broken"
		self.assertRaises(model.ConfigurationError, model.factory.generate_id,
			"irrelevant")

	def test_int(self):
		model.factory.auto_id_type = "int"
		p = model.Person()
		p2 = model.Activity()
		self.assertEqual(int(p.id[-1]), int(p2.id[-1])-1)

	def test_int_per_type(self):
		model.factory.auto_id_type = "int-per-type"
		p = model.Person()
		p2 = model.Person()
		self.assertEqual(int(p.id[-1]), int(p2.id[-1])-1)
		p3 = model.Activity()
		self.assertEqual(int(p.id[-1]), int(p3.id[-1]))		

	def test_int_per_segment(self):
		model.factory._auto_id_segments = {}
		model.factory.auto_id_type = "int-per-segment"
		model.Activity._uri_segment = model.Person._uri_segment
		p = model.Person()
		p2 = model.Activity()
		self.assertEqual(int(p.id[-1]), int(p2.id[-1])-1)		
		p3 = model.TimeSpan()
		self.assertEqual(int(p.id[-1]), int(p3.id[-1]))		
			
	def test_uuid(self):
		model.factory.auto_id_type = "uuid"
		p = model.Person()
		self.assertTrue(p.id.startswith('urn:uuid:'))		

	def test_prefixes(self):
		p = model.Person("ulan:1")
		self.assertEqual(p.id, "ulan:1")
		self.assertEqual(p._full_id, "http://vocab.getty.edu/ulan/1")

		p2 = model.Person('http://vocab.getty.edu/ulan/2')
		self.assertEqual(p2.id, 'ulan:2')
		self.assertEqual(p2._full_id, 'http://vocab.getty.edu/ulan/2')

		model.factory.prefixes = {'fish':'http://example.org/ns/'}
		p3 = model.Person('fish:3')
		self.assertEqual(p3.id, 'fish:3')
		self.assertEqual(p3._full_id, 'http://example.org/ns/3')

		model.factory.prefixes = {}
		p4 = model.Person('fish:4')
		self.assertTrue(p4.id.startswith(model.factory.base_url))

		
class TestBaseResource(unittest.TestCase):

	def setUp(self):
		self.artist = model.Person('00001', 'Jane Doe')
		self.son = model.Person('00002', 'John Doe')
		model.Person._properties['parent_of']['okayToUse'] = 1

	def test_init(self):
		self.assertEqual(self.artist.id, 'http://lod.example.org/museum/Person/00001')
		self.assertEqual(self.artist.type, 'crm:E21_Person')
		self.assertEqual(self.artist.label, 'Jane Doe')
		self.assertFalse(hasattr(self.artist, 'value'))
		self.assertFalse(hasattr(self.artist, 'has_type'))

	def test_check_prop(self):
		desc = self.artist._check_prop('label', 'Jane Doe\'s Bio')
		self.assertEqual(desc, 1)
		parent = self.artist._check_prop('parent_of', self.son)
		self.assertEqual(parent, 2)

	def test_list_all_props(self):
		props = self.artist._list_all_props()
		(lbl, cl) = sorted(props.items())[0]
		self.assertEqual('acquired_custody_through', lbl)
		self.assertEqual(model.TransferOfCustody, cl)

	def test_check_reference(self):
		self.assertTrue(self.artist._check_reference('http'))
		self.assertFalse(self.artist._check_reference('xxx'))
		self.assertTrue(self.artist._check_reference({'id': 'xxx'}))
		self.assertFalse(self.artist._check_reference({'xxx': 'yyy'}))
		self.assertTrue(self.artist._check_reference(self.son))
		self.assertTrue(self.artist._check_reference(['http']))
		self.assertFalse(self.artist._check_reference(['xxx', 'yyy']))
		self.assertTrue(self.artist._check_reference(model.Person))

	def test_multiplicity(self):
		model.factory.process_multiplicity = 1
		who = model.Actor()
		mmo = model.ManMadeObject()
		who.current_owner_of = mmo
		mmo.current_owner = who
		self.assertEqual(mmo.current_owner, who)
		self.assertEqual(who.current_owner_of, [mmo])		


class TestMagicMethods(unittest.TestCase):

	def setUp(self):
		model.Person._properties['parent_of']['okayToUse'] = 1
		model.Person._lang_properties = ['label', 'description']

	def test_set_magic_lang(self):
		model.factory.default_lang = 'en'
		artist = model.Person('00001', 'Jane Doe')
		self.assertEqual(artist.label, {'en': 'Jane Doe'})
		artist._set_magic_lang('label', 'Janey')
		self.assertEqual(artist.label, {'en': ['Jane Doe', 'Janey']})
		son = model.Person('00002', 'John Doe')
		self.assertRaises(model.DataError, artist._set_magic_lang, 'parent_of', son)

	def test_set_magic_resource(self):
		artist = model.Person('00001', 'Jane Doe')
		son = model.Person('00002', 'John Doe')
		daughter = model.Person('00002', 'Jenny Doe')
		son2 = model.Person('00002', 'Jim Doe')
		artist._set_magic_resource('parent_of', son)
		self.assertEqual(artist.parent_of, son)
		artist._set_magic_resource('parent_of', daughter)
		try:
			self.assertIn(son, artist.parent_of)
			self.assertIn(daughter, artist.parent_of)
		except:
			# 2.6 doesn't have assertIn
			self.assertTrue(son in artist.parent_of)
			self.assertTrue(daughter in artist.parent_of)

		artist._set_magic_resource('parent_of', son2)
		try:
			self.assertIn(son, artist.parent_of)
			self.assertIn(daughter, artist.parent_of)
			self.assertIn(son2, artist.parent_of)
		except:
			self.assertTrue(son in artist.parent_of)
			self.assertTrue(daughter in artist.parent_of)
			self.assertTrue(son2 in artist.parent_of)

	def test_set_magic_resource_inverse(self):
		model.factory.materialize_inverses = True
		artist = model.Person('00001', 'Jane Doe')
		son = model.Person('00002', 'John Doe')
		artist._set_magic_resource('parent_of', son)
		self.assertEqual(son.parent, artist)

	def test_validation_unknown(self):
		model.factory.validate_properties = True
		artist = model.Person('00001', 'Jane Doe')		
		self.assertRaises(model.DataError, artist.__setattr__, 'unknown_property', 1)

	def test_validation_wrong_type(self):
		model.factory.validate_properties = True
		artist = model.Person('00001', 'Jane Doe')	
		self.assertRaises(model.DataError, artist.__setattr__, 'parent_of', 'Bad Value')

	def test_validation_off(self):
		model.factory.validate_properties = False
		artist = model.Person('00001', 'Jane Doe')		
		artist.unknown_property = 1
		self.assertEqual(artist.unknown_property, 1)


if __name__ == '__main__':
	unittest.main()




from cidoc_orm import factory, Document, Activity, Event, TimeSpan, ManMadeObject, Acquisition, Type

# Locally "subclass" to create consistent patterns with E55 and AAT
class Painting(ManMadeObject):
	def __init__(self, *args, **kw):
		super(Painting, self).__init__(*args, **kw)
		self.has_type = Type("http://vocab.getty.edu/aat/300033618")

class LugtNumber(Identifier):
	def __init__(self, *args, **kw):
		super(LugtNumber, self).__init__(*args, **kw)
		# ???
		self.has_type = Type("http://vocab.getty.edu/aat/300033618")	

class TMSNumber(Identifier):
	def __init__(self, *args, **kw):
		super(TMSNumber, self).__init__(*args, **kw)
		# Repository Number
		self.has_type = Type("http://vocab.getty.edu/aat/300404621")

class LotNumber(Identifier):
	def __init__(self, *args, **kw):
		super(TMSNumber, self).__init__(*args, **kw)
		# Lot Number
		self.has_type = Type("http://vocab.getty.edu/aat/300404628")


# Or actually subclass in an extension vocab
class Mosaic(ManMadeObject):
	_type = "extension:Mosaic"

factory.base_url = "http://data.getty.edu/provenance/"
factory.default_lang = "en"

catalog = Document("catalog")
page = Document("catalog-entry")
catalog.has_component = page
auction = Activity("auction")
catalog.documents = auction
lot = Activity("lot")
auction.consists_of = lot
page.documents = lot
txn = Acquisition("sale")
lot.consists_of = txn
what = Painting('my-painting')
txn.transferred_title_of = what
what.label = "My First Paint By Numbers"
what.is_identified_by = TMSNumber("")


print factory.toString(catalog, compact=False)



import model
from model import CromulentFactory, build_classes, \
	KEY_ORDER_HASH, KEY_ORDER_DEFAULT

factory = CromulentFactory("http://lod.example.org/museum/", \
	load_context=False)
build_classes("utils/bibframe_vocab.tsv", top="rdf:Resource")
model.factory = factory



import json
import codecs
import re
import inspect

# for cidoc_orm, see: https://github.com/azaroth42/Python-CIDOC-ORM
from cidoc_orm import factory, TimeSpan, ManMadeObject, Type, Identifier, \
	Production, Actor, Person, Place, Group, Material, Mark, \
	Activity, InformationObject, Purchase, Acquisition, MonetaryAmount, \
	Currency, MeasurementUnit, Dimension, PhysicalObject

# Meta meta
ext_classes = {
	"LocalNumber": {"parent": Identifier, "vocab": "aat", "id": "300404621"},	
	"AccessionNumber": {"parent": Identifier, "vocab": "aat", "id": "300312355"},	
	"Inscription": {"parent": Mark, "vocab": "aat", "id": "300028702"},
	"Signature": {"parent": Mark, "vocab": "aat", "id": "300028705"},
	"Painting": {"parent": ManMadeObject, "vocab": "aat", "id": "300033618"},
	"Sculpture": {"parent": ManMadeObject, "vocab": "aat", "id": "300047090"},
	"Drawing": {"parent": ManMadeObject, "vocab": "aat", "id": "300033973"},
	"Miniature": {"parent": ManMadeObject, "vocab": "aat", "id": "300033936"},
	"Tapestry": {"parent": ManMadeObject, "vocab": "aat", "id": "300205002"},
	"Furniture": {"parent": ManMadeObject, "vocab": "aat", "id": "300037680"},
	"Mosaic": {"parent": ManMadeObject, "vocab": "aat", "id": "300015342"},
	"Photograph": {"parent": ManMadeObject, "vocab": "aat", "id": "300046300"},
	"Drawing": {"parent": ManMadeObject, "vocab": "aat", "id": "300033973"},
	"Coin": {"parent": ManMadeObject, "vocab": "aat", "id": "300037222"},
	"Vessel": {"parent": ManMadeObject, "vocab": "aat", "id": "300193015"},
	"PhotographPrint": {"parent": ManMadeObject, "vocab": "aat", "id": "300127104"},
	"PhotographAlbum": {"parent": ManMadeObject, "vocab": "aat", "id": "300026695"},
	"PhotographBook": {"parent": ManMadeObject, "vocab": "aat", "id": "300265728"}
}

factory.base_url = "http://data.getty.edu/provenance/"
factory.context_uri = "http://data.getty.edu/contexts/crm_context.jsonld"
 
for (name,v) in ext_classes.items():
	c = type(name, (v['parent'],), {})
	c._p2_has_type = "http://vocab.getty.edu/%s/%s" % (v['vocab'], v['id'])
	globals()[name] = c

# At the moment it's just an activity, no subtype info
class TakeInventory(Activity): 
	pass

class Payment(Activity):
	_properties = {
		"paid_amount": {"rdf": "pi:paid_amount", "range": MonetaryAmount},
		"paid_to": {"rdf": "pi:paid_to", "range": Actor},
		"paid_from": {"rdf": "pi:paid_from", "range": Actor}
	}
	_uri_segment = "Payment"
	_type = "pi:Payment"

Payment._classhier = inspect.getmro(Payment)[:-1]

# Object Types
# {'Pastel': 265, 'Photograph': 4, 'Clocks': 1, 'Painting [?]': 8, 'Painting': 37313, 
# '[not identified]': 2, 'Clothing': 1, 'Playing Cards': 1, 'Watercolor': 547, 
# 'Maps': 1, 'Book': 35, 'Decorative Art': 9, 'Painting; Sculpture': 1, 'Print': 21, 
# 'Watercolor; Painting': 1, 'Sculpture': 1817, 'Drawing': 225, 'Tapestry': 61, 'Furniture': 22}

endOfMonths = {'01': 31, '02': 28, '03':31, '04':30, '05':31, '06':30,\
	'07':31, '08':31, '09':30, '10':31, '11':30, '12':31}

aat_type_mapping = {
	"Painting": Painting,
	"Drawing": Drawing,
	"Furniture": Furniture,
	"Sculpture": Sculpture,
	"Tapestry": Tapestry,
	"Watercolor": Painting,
	"Pastel": Painting
}

#	  # A wooden support
aat_part_mapping = {
	"supports": "300014844"  # The thing that is painted on
}

aat_material_mapping = {
	"panel": "300014657",  # Is really a support
	"watercolor": "300015045",
	"oil": "300015050",
	"tempera": "300015062",
	"canvas": "300014078",
	"oak": "300012264",
	"gold leaf": "300264831",
	"paper": "300014109",
	"copper": "300011020",
	"terracotta": "300010669",
	"glass": "300010797",
	"chalk": "300011727",
	"bronze": "300010957",
	"marble": "300011443",
	"albumen silver print": "300127121",
	"gelatin silver print": "300128695",
	"silver": "300011029"
}

# pen, pencil, card, cardboard, porcelain, wax, ceramic, plaster
# crayon, millboard, gouache, brass, stone, lead, iron, clay,
# alabaster, limestone


materialTypes = {}
for (k,v) in aat_material_mapping.items():
	m = Material("http://vocab.getty.edu/aat/%s" % v)
	m.label = k
	materialTypes[k] = m

aat_culture_mapping = {
	"french": "300111188",
	"italian": "300111198",
	"german": "300111192",
	"dutch": "300020929"
}

dim_type_mapping = {
	"height": "300055644",
	"width": "300055647",
	"depth": "300072633",
	"diameter": "300055624",
	"weight": "300056240"
}

dim_unit_mapping = {
	"inches": "300379100",	
	"feet": "300379101",
	"cm": "300379098"
}

inches = MeasurementUnit("http://vocab.getty.edu/aat/%s" % dim_unit_mapping['inches']);
inches.label = "inches"

aat_genre_mapping = { 
	"Abstract" : "300134134", # maybe?
	"abstract" : "300134134", # maybe?
	"Genre": "300139140", # maybe?
	"History": "300386045",  
	"Landscape": "300015636",
	"Portrait": "300015637",
	"Still Life": "300015638"
}

genreTypes = {}
for (k,v) in aat_genre_mapping.items():
	t = Type("http://vocab.getty.edu/aat/%s" % v)
	t.label = k
	genreTypes[k] = t

aat_subject_mapping = {
	"Allegory": "",
	"Animals": "",
	"Architecture": "",
	"Battles": "",
	"Figure Studies": "",
	"Interiors": "",
	"Landscapes with figures": "",
	"Literature": "",
	"Marines": "",
	"Military": "",
	"Mythology (figures)": "",
	"Mythology (narrative)": "",
	"Religious (figures)": "",
	"Religious (narrative)": "",
	"Ruins": "",
	"Sporting": "",
	"Topographical Views": ""
}

# Monkey patch Type's _toJSON to only emit full data if not just URI+type
def typeToJSON(self, top=False):
	props = self.__dict__.keys()
	if len(props) > 3:
		return super(Type, self)._toJSON()
	else:
		return self.id

Type._toJSON = typeToJSON
Person._properties['familyName'] = {"rdf": "schema:familyName", "range": str}
Person._properties['givenName'] = {"rdf": "schema:givenName", "range": str}
Person._properties['nationality'] = {"rdf": "schema:nationality", "range": Place}
ManMadeObject._properties['culture'] = {"rdf": "schema:genre", "range": Type}
ManMadeObject._properties['height'] = {"rdf": "schema:height", "range": Dimension}
ManMadeObject._properties['width'] = {"rdf": "schema:width", "range": Dimension}

knoedler = Group("knoedler")
knoedler.label = "Knoedler"

factory.materialize_inverses = True

def process_money_value(value):
	value = value.replace('[', '')
	value = value.replace(']', '')
	value = value.replace('-', '')
	value = value.replace("?", '')
	value = value.replace('not written', '')
	value = value.replace('sold at auction', '')
	value = value.replace('x', '')
	value = value.replace('Lot Price', '')
	value = value.replace('See sales book', '')
	value = value.strip()
	return value

def process_materials(what, materials):
	materials = materials.lower()
	materials = materials.replace("&", "and")
	materials = materials.replace("card [cardboard]", "cardboard")
	materials = materials.replace("c [canvas]", "canvas")
	materials = materials.replace("w/c [watercolor]", "watercolor")
	materials = materials.replace("[bronze]", "bronze")
	materials = materials.replace("c on p [canvas on panel]", "canvas on panel")
	materials = materials.replace("[from sales book 9, 1907-1912, f. 361]", "")
	materials = materials.replace("p [panel]", "panel")
	materials = materials.replace('terra cotta', 'terracotta')
	materials = materials.replace(',', '')
	materials = materials.replace('-', '')
	materials = materials.replace('procelain', 'porcelain')
	materials = materials.strip()
	matwds = materials.split(' ')
	mats = []
	for mw in matwds:
		if mw in ['on', 'and']:
			continue
		try:
			mats.append(materialTypes[mw])
		except:
			# print "Material not found: %s" % mw
			pass

	return mats

divre = re.compile('^([0-9]+) ([0-9]+)/([0-9]+)( |$)')
unitre = re.compile('^([0-9.]+) (high|height|h|long|length|l)( |$)')
	
def process_dimensions(dims):
	dims = dims.lower()
	# assume default of inches
	dims = dims.replace('"', '')
	dims = dims.replace('in.', '')
	dims = dims.replace('inches', '')
	dims = dims.replace('//', '/')
	dims = dims.replace('[', '')
	dims = dims.replace(']', '')
	dims = dims.replace('X', 'x')
	dl = dims.split('x')

	dimensions = []
	p = 0
	for d in dl:
		d = d.strip()
		# check for (nn n/n)
		which = ""
		m = divre.match(d)
		if m:
			(main, numr, denom, end) = m.groups()
			ttl = int(main) + (float(numr) / float(denom))
		else:
			try:
				ttl = int(d)
			except:
				m = unitre.match(d)
				if m:
					(ttl, which, end) = m.groups()
					if which.startswith('h'):
						which = 'h'
					else:
						which = 'w'
				else:
					# print "----- %s" % d
					continue
		if not which:
			which = "w" if p else "h"
		p += 1
		dimensions.append([ttl, which])
	return dimensions


def print_rec_full(rec):
	its = rec.items()
	its.sort()
	for (k,v) in its:
		if v:
			print "%s: %s" % (k, v)	

stock_books = {}
pages = {}

fh = file('knoedler_cache.json')
cache = json.load(fh)
fh.close()

recs = cache.values()
recs = sorted(recs, key=lambda x: x['star_id'])[:5000]
# recs = [cache['72910']]

for rec in recs:

	bookId = rec['stock_book_id']
	try:
		book = stock_books[bookId]
	except:
		# create the book
		book = InformationObject(bookId)
		book.label = "Knoedler Stock Book %s" % bookId
		stock_books[bookId] = book

	pageId = "%s/%s" % (bookId, rec['page_num'])
	try:
		page = pages[pageId]
	except:
		# create the page in the book
		page = InformationObject(pageId)
		page.label = "Page %s" % rec['page_num']
		pages[pageId] = page
		book.has_fragment = page

	# create the entry
	entryId = "%s/%s" % (pageId, rec['row_num'])
	entry = InformationObject(entryId)
	entry.label = "Row %s" % rec['row_num']
	page.has_fragment = entry

	# the description and notes fields are related to the entry
	# not the object
	if rec['notes']:
		entry.description = rec['notes']
	if rec['description']:
		entry.description = rec['description']

	oid = rec['pi_id']

	# create the activity that the entry describes
	# 'Sold':26598,'Unsold':11824,'Exchanged':103,'Presented':246,'Transferred':310
	# 'Returned':533,'Unknown': 629,   
	# 'Lost': 7, 'Voided': 16, 'Disjointed': 16, 'Cancelled': 47, 'Removed': 6

	# The outbound activity
	txn = None
	inv = None

	txnType = rec['transaction']
	if txnType == "Sold":
		txn = Purchase(oid)
	elif txnType in ['Exchanged', 'Presented', 'Transferred']:
		if rec['price_amount']:
			txn = Purchase(oid)
		else:
			txn = Acquisition(oid)
	elif txnType in ["Unsold", "Cancelled"]:
		# Stock taking, or never left inventory due to no sale
		inv = TakeInventory(oid)
	elif txnType in ["Lost", "Removed"]:
		# Leaves inventory, but not via a transfer of ownership
		# E8 can represent end of ownership. There's just no new owner.
		txn = Acquisition(oid)
	elif txnType == "Voided":
		# Bad data; voided should be skipped (per Kelly)
		continue
	elif txnType == "Returned":
		# Can't tell what this actually means yet
		# Could be entering or leaving Knoedler stock!
		continue
	elif txnType == "Unknown":
		if rec['price_amount']:
			txn = Purchase(oid)
		else:
		 inv = TakeInventory(oid)
	else:
		# I think this is only Disjointed
		# print_rec(rec)
		continue

	# The inbound activity that always happens
	# For consistency, always generate a Payment
	if rec['purchase_amount']:
		inTxn = Purchase("purch_%s" % oid)
		pay = Payment("purch_%s" % oid)
		inTxn.consists_of = pay

		amnt = MonetaryAmount("purch_price_%s" % oid)
		value = process_money_value(rec['purchase_amount'])
		if value:
			try:
				amnt.has_value = float(value)
			except:
				amnt.description = value
		if rec['purchase_currency']:
			curr = Currency(rec['purchase_currency'])
			curr.label = rec['purchase_currency']
			amnt.has_currency = curr
		if rec['purchase_note']:
			amnt.description = rec['purchase_note']		
		pay.paid_amount = amnt
		pay.paid_from = knoedler
		inTxn.had_sales_price = amnt

	else:
		inTxn = Acquisition("purch_%s" % oid)

	inTxn.transferred_title_to = knoedler
	if rec['seller_name'] or rec['seller_name_auth']:
		# Look up in authority?
		seller = Actor("seller_%s" % oid)
		seller.label = rec['seller_name_auth'] if rec['seller_name_auth'] else rec['seller_name']
		if rec['seller_loc'] or rec['seller_loc_auth']:
			sellerPlace = Place("seller_place_%s" % oid)
			sellerPlace.label = rec['seller_loc_auth'] if rec['seller_loc_auth'] else rec['seller_loc']
			seller.has_current_or_former_residence = sellerPlace
		inTxn.transferred_title_from = seller
		if rec['purchase_amount']:
			pay.paid_to = seller

	# CurationPeriod
	curated = Activity("curated_%s" % oid)
	curated.is_started_by = inTxn

	if txn:
		# from
		txn.transferred_title_from = knoedler
		# to
		if rec['buyer_name'] or rec['buyer_name_auth']:
			# Look up in authority?
			buyer = Actor("buyer_%s" % oid)
			buyer.label = rec['buyer_name_auth'] if rec['buyer_name_auth'] else rec['buyer_name']
			if rec['buyer_loc'] or rec['buyer_loc_auth']:
				buyerPlace = Place("buyer_place_%s" % oid)
				buyerPlace.label = rec['buyer_loc_auth'] if rec['buyer_loc_auth'] else rec['buyer_loc']
				buyer.has_current_or_former_residence = buyerPlace
			txn.transferred_title_to = buyer

		# when
		if rec['sale_date_year']:
			# if year, then all. blank is "00"			
			yr = rec['sale_date_year']
			mt = rec['sale_date_month']
			dy = rec['sale_date_day']
			if dy != "00":
				start = "%s-%s-%s" % (yr,mt,dy)
				end = start
			elif mt != "00":
				start = "%s-%s-01" % (yr,mt)
				end = "%s-%s-%s" % (yr,mt,endOfMonths[mt])
			else:
				start = "%s-01-01" % yr
				end = "%s-12-31" % yr
			span = TimeSpan("sale_span_%s" % oid)
			span.begin_of_the_begin = start
			span.end_of_the_end = end
			txn.has_timespan = span

		value = process_money_value(rec['price_amount'])
		if value:
			amnt = MonetaryAmount("sale_price_%s" % oid)
			try:
				amnt.has_value = float(value)
			except:
				amnt.description = value
			if rec['price_currency']:
				curr = Currency(rec['price_currency'])
				curr.label = rec['price_currency']
				amnt.has_currency = curr
			if rec['price_note']:
				amnt.description = rec['price_note']
			txn.had_sales_price = amnt

			# Check knoedler_share
			if rec['knoedler_share_amount']:

				value = process_money_value(rec['knoedler_share_amount'])
				if value:
					amnt = MonetaryAmount("shared_price_%s" % oid)
					try:
						amnt.has_value = float(value)
					except:
						amnt.description = value
					if rec['knoedler_share_currency']:
						curr = Currency(rec['knoedler_share_currency'])
						curr.label = rec['knoedler_share_currency']
						amnt.has_currency = curr
					if rec['knoedler_share_note']:
						amnt.description = rec['knoedler_share_note']

				pay = Payment("kshare_%s" % oid)
				txn.consists_of = pay
				pay.paid_amount = amnt
				pay.paid_to = knoedler
				if rec['buyer_name'] or rec['buyer_name_auth']:
					pay.paid_from = buyer

			else:
				pay = Payment("sale_%s" % oid)
				txn.consists_of = pay
				pay.paid_amount = amnt
				pay.paid_to = knoedler
				if rec['buyer_name'] or rec['buyer_name_auth']:
					pay.paid_from = buyer

		curated.is_finished_by = txn
	elif inv:
		# Taking of Inventory as part of the curation period
		curated.consists_of = inv

		# If taking inventory, then the entry date is for that
		if rec['entry_date_year']:
			# if year, then all. blank is "00"			
			yr = rec['entry_date_year']
			mt = rec['entry_date_month']
			dy = rec['entry_date_day']
			if dy != "00":
				start = "%s-%s-%s" % (yr,mt,dy)
				end = start
			elif mt != "00":
				start = "%s-%s-01" % (yr,mt)
				end = "%s-%s-%s" % (yr,mt,endOfMonths[mt])
			else:
				start = "%s-01-01" % yr
				end = "%s-12-31" % yr
			span = TimeSpan("sale_span_%s" % oid)
			span.begin_of_the_begin = start
			span.end_of_the_end = end
			inv.has_timespan = span


	if not inv:
		# entry date is for purchase
		if rec['entry_date_year']:
			# if year, then all. blank is "00"			
			yr = rec['entry_date_year']
			mt = rec['entry_date_month']
			dy = rec['entry_date_day']
			if dy != "00":
				start = "%s-%s-%s" % (yr,mt,dy)
				end = start
			elif mt != "00":
				start = "%s-%s-01" % (yr,mt)
				end = "%s-%s-%s" % (yr,mt,endOfMonths[mt])
			else:
				start = "%s-01-01" % yr
				end = "%s-12-31" % yr
			span = TimeSpan("purch_span_%s" % oid)
			span.begin_of_the_begin = start
			span.end_of_the_end = end
			inTxn.has_timespan = span

	# create the object of the transaction

	objectType = rec['object_type']
	try:
		what = aat_type_mapping[objectType](oid)
	except:
		what = ManMadeObject(oid)

	curated.used_specific_object = what
	inTxn.transferred_title_of = what
	entry.refers_to = inTxn
	if txn:
		txn.transferred_title_of = what
		entry.refers_to = txn
	elif txnType == "Voided":
		entry.refers_to = what	

	what.label = rec['title']

	idnt = AccessionNumber("knoedler_%s" % oid)
	idnt.value = rec['knoedler_id']
	# No way to say it's Knoedler's number?
	# Could have a Creation of the Identifier performed by Knoedler :(

	if rec['artist_name'] or rec['artist_name_auth']:
		artist = Person("artist_%s" % oid)
		artist.label = rec['artist_name_auth'] if rec['artist_name_auth'] else rec['artist_name']
		if rec['nationality']:
			artist.nationality = Place("artist_natl_%s" % oid)
			artist.nationality.label = rec['nationality']

		prodn = Production("production_%s" % oid)
		prodn.carried_out_by = artist
		what.was_produced_by = prodn

	if rec['artist_name_2'] or rec['artist_name_auth_2']:
		artist = Person("artist2_%s" % oid)
		artist.label = rec['artist_name_auth_2'] if rec['artist_name_auth_2'] else rec['artist_name_2']
		if rec['nationality_2']:
			artist.nationality = Place('artist_2_natl_%s' % oid)
			artist.nationality.label = rec['nationality_2']
		prodn.carried_out_by = artist


	# genre
	if rec['genre'] and not rec['genre'] == '[not identified]':
		if not aat_genre_mapping.has_key(rec['genre']):
			print "Not found: %s" % (rec['genre'])
		else:	
			what.has_type = genreTypes[rec['genre']]

	# subject
	if rec['subject']:
		s = rec['subject']
		if s.find(';'):
			ss = [x.strip() for x in s.split(';')]
		else:
			ss = [s]
		for s in ss:
			# s  = s.replace('Int\xe9rieurs', 'Interiors')
			sid = s.replace(' ', '')
			sid = sid.replace('(', '')
			sid = sid.replace(')', '')
			t = Type(sid)
			t.label = s
			what.depicts = t

	# materials
	if rec['materials']:
		# XXX Finish this
		process_materials(what, rec['materials'])
		# what.made_of = material

	if rec['dimensions']:
		# XXX Finish this too
		dims = process_dimensions(rec['dimensions'])
		for d in dims:
			dim = Dimension("%s_%s" % (d[0], oid))
			dim.has_value = d[0]
			dim.has_unit = inches
			if d[1] == 'h':
				what.height = dim
			else:
				what.width = dim				

collection = InformationObject("collection")
for s in stock_books.values():
	collection.has_fragment = s


factory.full_names = True
outstr = factory.toString(collection, compact=False)

fh = file('knoedler.jsonld', 'w')
fh.write(outstr)
fh.close()

# Note that these entries are really one transaction
# 64699 ... 64732




import model
from model import CromulentFactory, build_classes, \
	KEY_ORDER_HASH, KEY_ORDER_DEFAULT

factory = CromulentFactory("http://lod.example.org/museum/", \
	context="http://linked.art/ns/context/1/full.jsonld")
build_classes()
model.factory = factory


 
# This assumes the default CIDOC-CRM, even though the model code 
# can generate classes for any ontology

from .model import Identifier, Mark, ManMadeObject, Type, \
	Person, Material, MeasurementUnit, Place, Dimension, Currency, \
	ConceptualObject, TimeSpan, Actor, PhysicalThing, Language, \
	LinguisticObject, InformationObject, SpatialCoordinates, \
	Activity, Group, Name, MonetaryAmount, PropertyInterest, \
	Destruction, AttributeAssignment, BaseResource, PhysicalObject, \
	Acquisition, ManMadeFeature, VisualItem, Aggregation, Proxy, \
	PropositionalObject, Payment, Creation, ContactPoint, Phase, \
	STR_TYPES, factory

# Add classified_as initialization hack for all resources
def post_init(self, **kw):
	if self.__class__._classification:
		self.classified_as = self._classification
BaseResource._post_init = post_init

def register_aat_class(name, parent, id, label):
	c = type(name, (parent,), {})
	if id.startswith('http'):
		t = Type(id)
	else:
		t = Type("http://vocab.getty.edu/aat/%s" % id)
	t.label = label
	c._classification = t
	c._type = None # To avoid conflicting with parent class
	globals()[name] = c	

instances = {}

def register_instance(name, parent, id, label):
	if id.startswith('http'):
		t = parent(id)
	else:
		t = parent("http://vocab.getty.edu/aat/%s" % id)
	t.label = label
	instances[name] = t

# Meta meta
ext_classes = {
	"LocalNumber": {"parent":Identifier, "id":"300404621", "label": "Local Number"},	
	"AccessionNumber": {"parent":Identifier, "id":"300312355", "label": "Accession Number"},
	"LotNumber": {"parent": Identifier, "id": "300404628", "label": "Lot Number"},

	"DigitalImage": {"parent": VisualItem, "id": "300215302", "label": "Digital Image"},

	"OwnershipRight": {"parent": PropertyInterest, "id":"300055603", "label": "Ownership Right"},
	"CustodyRight": {"parent": PropertyInterest, "id":"300411616", "label": "Custody Right"},
	"CopyrightRight": {"parent": PropertyInterest, "id":"300055598", "label": "Copyright"},

	"Inscription": {"parent": LinguisticObject, "id": "300028702", "label": "Inscription"},
	"Signature": {"parent": LinguisticObject, "id": "300028705", "label": "Signature"},
	"MaterialStatement": {"parent": LinguisticObject, "id": "300010358", "label": "Material Statement"},
	"DimensionStatement": {"parent": LinguisticObject, "id": "300266036", "label": "Dimension Statement"},
	"CreditStatement": {"parent": LinguisticObject, "id": "300026687", "label": "Credit Statement"},
	"RightsStatement": {"parent": LinguisticObject, "id": "300055547", "label": "Rights Statement"},
	"EditionStatement": {"parent": LinguisticObject, "id":"300121294", "label": "Edition Statement"},
	"BiographyStatement": {"parent": LinguisticObject, "id":"300080102", "label": "Biography Statement"},
	"ProvenanceStatement": {"parent": LinguisticObject, "id":"300055863", "label": "Provenance Statement"},
	"Description": {"parent": LinguisticObject, "id":"300080091", "label": "Description"},
	"PaginationStatement": {"parent": LinguisticObject, "id":"300200294", "label": "Pagination Statement"},
	"FoliationStatement": {"parent": LinguisticObject, "id":"300200662", "label": "Foliation Statement"},
	"Abstract": {"parent": LinguisticObject, "id":"300026032", "label": "Abstract"},

	"CatalogueRaisonne": {"parent": LinguisticObject, "id":"300026061", "label": "Catalogue Raisonne"},
	"AuctionCatalog": {"parent": LinguisticObject, "id":"300026068", "label": "Auction Catalog"},
	"SalesCatalog": {"parent": LinguisticObject, "id": "300026074", "label": "Sales Catalog"},
	"ExhibitionCatalog": {"parent": LinguisticObject, "id": "300026096", "label": "Exhibition Catalog"},
	"AccountBook": {"parent": LinguisticObject, "id": "300027483", "label": "Account Book"},
	"WebPage": {"parent": LinguisticObject, "id":"300264578", "label": "Web Page"},
	"Register": {"parent": LinguisticObject, "id":"300027168", "label": "Register"},

	"Page": {"parent": LinguisticObject, "id":"300194222", "label": "Page"},
	"Folio": {"parent": LinguisticObject, "id":"300189604", "label": "Folio"},
	"DataRecord": {"parent": LinguisticObject, "id":"300026685", "label": "Data Record"}, # Not sure about this one

	"Journal": {"parent": LinguisticObject, "id":"300215390", "label": "Journal"},
	"Issue": {"parent": LinguisticObject, "id":"300312349", "label": "Issue"},
	"Volume": {"parent": LinguisticObject, "id":"300265632", "label": "Volume"},
	"Article": {"parent": LinguisticObject, "id":"300048715", "label": "Article"},
	"Chapter": {"parent": LinguisticObject, "id":"300311699", "label": "Chapter"},
	"Series": {"parent": LinguisticObject, "id":"300027349", "label": "Series"},
	"Proceedings": {"parent": LinguisticObject, "id":"300027316", "label": "Proceedings"},
	"Monograph": {"parent": LinguisticObject, "id":"300060417", "label": "Monograph"},
	"Patent": {"parent": LinguisticObject, "id":"300027832", "label": "Patent"},
	"Thesis": {"parent": LinguisticObject, "id":"300028028", "label": "Thesis"},
	"TechnicalReport": {"parent": LinguisticObject, "id":"300027323", "label": "Technical Report"},
	"DocManuscript": {"parent": LinguisticObject, "id":"300028579", "label": "Document Manuscript"},
	"AudioVisualContent": {"parent": LinguisticObject, "id":"300028045", "label": "A/V Content"},

	"Height":     {"parent": Dimension, "id":"300055644", "label": "Height"},
	"Width":      {"parent": Dimension, "id":"300055647", "label": "Width"},
	"Depth":      {"parent": Dimension, "id":"300072633", "label": "Depth"},
	"Diameter":   {"parent": Dimension, "id":"300055624", "label": "Diameter"},
	"Weight":     {"parent": Dimension, "id":"300056240", "label": "Weight"},
	"Color":      {"parent": Dimension, "id":"300056130", "label": "Color"},

	"Gallery":      {"parent": Place, "id":"300240057", "label": "Gallery"},
	"AuctionHouse": {"parent": Place, "id":"300005234", "label": "Auction House"},
	"MuseumPlace":  {"parent": Place, "id":"300005768", "label": "Museum"},
	"ExhibitionPlace": {"parent": Place, "id":"300005748", "label": "Exhibition"},

	"MuseumOrg":   {"parent": Group, "id":"300312281", "label": "Museum"},
	"Department":  {"parent": Group, "id":"300263534", "label": "Department"},
	"Nationality": {"parent": Aggregation, "id":"300379842", "label": "Nationality"},
	"Gender":      {"parent": Aggregation, "id":"300055147", "label": "Gender"},

	"Auctioneer":  {"parent": Person, "id":"300025208", "label": "Auctioneer"}, # is this useful?

	"Auction":     {"parent": Activity, "id":"300054751", "label": "Auctioning"},
	"Bidding":     {"parent": Creation, "id":"300054602", "label": "Bidding"}, # individual bid
	"Curating":    {"parent": Activity, "id":"300054277", "label": "Curating"},
	"Inventorying": {"parent": Activity, "id":"300077506", "label": "Inventorying"},
	"Provenance":  {"parent": Activity, "id":"300055863", "label": ""},
	"Exhibition":  {"parent": Activity, "id":"300054766", "label": "Exhibiting"},
	"MultiExhibition": {"parent": Activity, "id":"300054773", "label": "Exhibiting in multiple locations"},
	"Active":      {"parent": Activity, "id":"300393177", "label": "Professional Activities"},
	"Publishing":  {"parent": Activity, "id":"300054686", "label": "Publishing"},
	"Purchase":  {"parent":Acquisition, "id":"300077989", "label": "Puchasing"},

	"ExhibitionIdea": {"parent": PropositionalObject, "id":"300417531", "label": "Exhibition"},

	"Theft": {"parent": Acquisition, "id": "300055292", "label": "Theft"},
	"Looting": {"parent": Acquisition, "id":"300379554", "label": "Looting"},

	"SupportPart": {"parent": ManMadeObject, "id":"300014844", "label": "Support"},
	"FramePart": {"parent": ManMadeObject, "id":"300404391", "label": "Frame"},
	"MountPart": {"parent": ManMadeObject, "id":"300131087", "label": "Mount"},
	"PanelPart": {"parent": ManMadeObject, "id":"300014657", "label": "Panel"},

	"AuctionLotSet": {"parent": Aggregation, "id":"300411307", "label": "Auction Lot"},
	"CollectionSet": {"parent": Aggregation, "id":"300025976", "label": "Collection"},

	"PrimaryName": {"parent": Name, "id":"300404670", "label": "Primary Name"},
	"SortName": {"parent": Name, "id":"300404672", "label": "Sorting Name"},
	"PersonalName": {"parent": Name, "id":"300266386", "label": "Personal Name"},
	"Title": {"parent": Name, "id":"300195168", "label": "Title"},
	"DisplayName": {"parent": Name, "id": "300404669", "label": "Display Title"},
	"AliasName": {"parent": Name, "id": "300404664", "label": "Alias"},
	"MaidenName": {"parent": Name, "id": "300404682", "label": "Maiden Name"},

	"GivenName": {"parent": Name, "id":"300404651", "label": "Given Name"},
	"FamilyName": {"parent": Name, "id":"300404652", "label": "Family Name"},
	"MiddleName": {"parent": Name, "id":"300404654", "label": "Middle Name"},
	"NameSuffix": {"parent": Name, "id":"300404662", "label": "Name Suffix"},
	"NamePrefix": {"parent": Name, "id":"300404845", "label": "Name Prefix"},

	"EmailAddress": {"parent": ContactPoint, "id":"300149026", "label": "Email Address"},
	"StreetAddress": {"parent": ContactPoint, "id":"300386983", "label": "Street Address"},
	"StreetNumber": {"parent": ContactPoint, "id":"300419272", "label": "Street Number"},
	"StreetName": {"parent": ContactPoint, "id": "300419273", "label": "Street Name"},
	"PostalCode": {"parent": ContactPoint, "id": "300419274", "label": "Postal Code"},

	"Painting": {"parent": ManMadeObject, "id": "300033618", "label": "Painting"},
	"Sculpture": {"parent": ManMadeObject, "id": "300047090", "label": "Sculpture"},
	"Drawing": {"parent": ManMadeObject, "id": "300033973", "label": "Drawing"},
	"Miniature": {"parent": ManMadeObject, "id": "300033936", "label": "Miniature"},
	"Tapestry": {"parent": ManMadeObject, "id": "300205002", "label": "Tapestry"},
	"Furniture": {"parent": ManMadeObject, "id": "300037680", "label": "Furniture"},
	"Mosaic": {"parent": ManMadeObject, "id": "300015342", "label": "Mosaic"},
	"Photograph": {"parent": ManMadeObject, "id": "300046300", "label": "Photograph"},
	"Coin": {"parent": ManMadeObject, "id": "300037222", "label": "Coin"},
	"Vessel": {"parent": ManMadeObject, "id": "300193015", "label": "Vessel"},
	"Graphic": {"parent": ManMadeObject, "id": "300015387", "label": "Graphic Art"},
	"Enamel": {"parent": ManMadeObject, "id": "300178264", "label": "Enamel"},
	"Embroidery": {"parent": ManMadeObject, "id": "300264024", "label": "Embroidery"},
	"PhotographPrint": {"parent": ManMadeObject, "id": "300127104", "label": "Photograph Print"},
	"PhotographAlbum": {"parent": ManMadeObject, "id": "300026695", "label": "Photograph Album"},
	"PhotographBook": {"parent": ManMadeObject, "id": "300265728", "label": "Photograph Book"},
	"PhotographColor": {"parent": ManMadeObject, "id": "300128347", "label": "Color Photograph"},
	"PhotographBW": {"parent": ManMadeObject, "id": "300128359", "label": "Black and White Photograph"},
	"Negative": {"parent": ManMadeObject, "id": "300127173", "label": "Photographic Negative"},

	"Architecture": {"parent": ManMadeObject, "id":"300263552", "label": "Architecture"},
	"Armor": {"parent": ManMadeObject, "id":"300036745", "label": "Armor"},
	"Book": {"parent": ManMadeObject, "id":"300028051", "label": "Book"},
	"DecArts": {"parent": ManMadeObject, "id":"300054168", "label": "Decorative Arts"},
	"Implement": {"parent": ManMadeObject, "id":"300024841", "label": "Implement"},
	"Jewelry": {"parent": ManMadeObject, "id":"300209286", "label": "Jewelry"},
	"Manuscript": {"parent": ManMadeObject, "id":"300265483", "label": "Manuscript"},
	"SiteInstallation": {"parent": ManMadeObject, "id":"300047896", "label": "Site Installation"},
	"Text": {"parent": ManMadeObject, "id":"300263751", "label": "Text"},
	"Print":  {"parent": ManMadeObject, "id":"300041273", "label": "Print"},
	"TimeBasedMedia": {"parent": ManMadeObject, "id":"300185191", "label": "Time-based Media"},

	"FrontPart": {"parent": ManMadeObject, "id":"300190703", "label": "Front Part"},
	"BackPart": {"parent": ManMadeObject, "id":"300190692", "label": "Back Part"},
	"TopPart": {"parent": ManMadeObject, "id":"300190710", "label": "Top Part"},
	"BottomPart": {"parent": ManMadeObject, "id":"300190695", "label": "Bottom Part"},
	"SidePart": {"parent": ManMadeObject, "id":"300190706", "label": "Side Part"},

	"StartingPrice": {"parent": MonetaryAmount, "id": "300417242", "label": "Starting Price"},
	"ReservePrice": {"parent": MonetaryAmount, "id": "300417243", "label": "Reserve Price"},
	"EstimatedPrice": {"parent": MonetaryAmount, "id": "300417244", "label": "Estimated Price"},

	"CommissionPayment": {"parent": Payment, "id": "300393199", "label": "Commission"}

}
 
for (name,v) in ext_classes.items():
	register_aat_class(name, v['parent'], v['id'], v['label'])

identity_instances = {
	"watercolor": {"parent": Material, "id": "300015045", "label": "watercolors"},	
	"oil": {"parent": Material, "id": "300015050", "label": "oil"},
	"tempera": {"parent": Material, "id": "300015062", "label": "tempera"},
	"canvas": {"parent": Material, "id": "300014078", "label": "canvas"},
	"oak": {"parent": Material, "id": "300012264", "label": "oak"},
	"gold leaf": {"parent": Material, "id": "300264831", "label": "gold leaf"},
	"paper": {"parent": Material, "id": "300014109", "label": "paper"},
	"copper": {"parent": Material, "id": "300011020", "label": "copper"},
	"terracotta": {"parent": Material, "id": "300010669", "label": "terracotta"},
	"glass": {"parent": Material, "id": "300010797", "label": "glass"},
	"chalk": {"parent": Material, "id": "300011727", "label": "chalk"},
	"bronze": {"parent": Material, "id": "300010957", "label": "bronze"},
	"marble": {"parent": Material, "id": "300011443", "label": "marble"},
	"albumen silver print": {"parent": Material, "id": "300127121", "label": "albumen silver print"},
	"gelatin silver print": {"parent": Material, "id": "300128695", "label": "gelatin silver print"},
	"silver": {"parent": Material, "id": "300011029", "label": "silver"},

	"inches": {"parent": MeasurementUnit, "id": "300379100", "label": "inches"},
	"feet": {"parent": MeasurementUnit, "id":"300379101", "label": "feet"},
	"cm": {"parent": MeasurementUnit, "id": "300379098", "label": "centimeters"},
	"meters": {"parent": MeasurementUnit, "id": "300379099", "label": "meters"},

	"english": {"parent": Language, "id": "300388277", "label": "English"},
	"french": {"parent": Language, "id":"300388306","label": "French"},
	"german": {"parent": Language, "id":"300388344","label": "German"},
	"spanish": {"parent": Language, "id":"300389311","label": "Spanish"},
	"chinese": {"parent": Language, "id":"300388113", "label":"Chinese"},
	"hindi": {"parent": Language, "id":"300388412", "label":"Hindi"},
	"arabic": {"parent": Language, "id":"300387843", "label":"Arabic"},
	"portuguese":{"parent": Language, "id":"300389115", "label":"Portuguese"},
	"bengali": {"parent": Language, "id":"300387971", "label":"Bengali"},
	"russian": {"parent": Language, "id":"300389168", "label":"Russian"},
	"dutch": {"parent": Language, "id":"300388256", "label":"Dutch"},
	"italian": {"parent": Language, "id":"300388474", "label":"Italian"},
	"greek": {"parent": Language, "id":"300389734", "label":"Greek"},
	"ancient greek": {"parent": Language, "id": "300387827", "label": "Ancient Greek"},
	"latin": {"parent": Language, "id":"300388693", "label":"Latin"},
	"japanese": {"parent": Language, "id":"300388486", "label":"Japanese"},
	"primary": {"parent": Type, "id": "300404670", "label": "Primary"},
	"artwork": {"parent": Type, "id": "300133025", "label": "Artwork"},
	"public collection": {"parent": Type, "id": "300411912", "label": "Public Collection"},
	"style of": {"parent": Type, "id": "300404285", "label": "Style Of"},
	"computer generated": {"parent": Type, "id": "300202389", "label": "Computer Generated"},
	"gender issues": {"parent": Type, "id": "300233686", "label": "Gender Issues"},
	"war": {"parent": Type, "id": "300055314", "label": "War"},
	"impressionism": {"parent": Type, "id":"300021503", "label": "Impressionism"},
	"landscapes": {"parent": Type, "id":"300015636", "label": "Landscapes"},
	"oval": {"parent": Type, "id":"300263817", "label": "Oval"},
	"city": {"parent": Type, "id": "300008389", "label": "City"},
	"province": {"parent": Type, "id":"300000774", "label": "Province"},
	"nation": {"parent": Type, "id": "300128207", "label": "Nation"},
	"building": {"parent": Type, "id": "300004792", "label": "Building"},
	"city block": {"parent": Type, "id": "300008077", "label": "City Block"},
	"crime": {"parent": Type, "id": "300055289", "label": "Crime"},
	"glassblowing": {"parent": Type, "id":"300053932", "label":"Glassblowing"},
	"sculpting": {"parent": Type, "id":"300264383", "label": "Sculpting"},
	"painting": {"parent": Type, "id":"300161986", "label": "Painting"},
	"first": {"parent": Type, "id":"300404050", "label": "First"},
	"last": {"parent": Type, "id": "XXX", "label": "Last"},

	"french nationality": {"parent": Type, "id": "300111188", "label": "French"},
	"american nationality": {"parent": Type, "id": "300107956", "label": "American"},
	"italian nationality": {"parent": Type, "id": "300111198", "label": "Italian"},
	"dutch nationality": {"parent": Type, "id": "300111175", "label": "Dutch"},
	"belgian nationality": {"parent": Type, "id": "300111156", "label": "Belgian"},
	"british nationality": {"parent": Type, "id": "300111159", "label": "British"},	
	"flemish nationality": {"parent": Type, "id": "300111184", "label": "Flemish"},
	"german nationality": {"parent": Type, "id": "300111192", "label": "German"},
	"austrian nationality": {"parent": Type, "id": "300111153", "label": "Austrian"},
	"spanish nationality": {"parent": Type, "id": "300111215", "label": "Spanish"},
	"swiss nationality": {"parent": Type, "id": "300111221", "label": "Swiss"},
	"irish nationality": {"parent": Type, "id": "300111259", "label": "Irish"},
	"hungarian nationality": {"parent": Type, "id": "300111195", "label": "Hungarian"},
	"swedish nationality": {"parent": Type, "id": "300111218", "label": "Swedish"},
	"czech nationality": {"parent": Type, "id": "300111166", "label": "Czech"},
	"russian nationality": {"parent": Type, "id": "300111276", "label": "Russian"},
	"polish nationality": {"parent": Type, "id": "300111204", "label": "Polish"},
	"norwegian nationality": {"parent": Type, "id": "300111201", "label": "Norwegian"},
	"danish nationality": {"parent": Type, "id": "300111172", "label": "Danish"},
	"chinese nationality": {"parent": Type, "id": "300018322", "label": "Chinese"},
	"egyptian nationality": {"parent": Type, "id": "300020251", "label": "Egyptian"},
	"greek nationality": {"parent": Type, "id": "300264816", "label": "Greek"},
	"canadian nationality": {"parent": Type, "id": "300107962", "label": "Canadian"},
	"mexican nationality": {"parent": Type, "id": "300107963", "label": "Mexican"},
	"portuguese nationality": {"parent": Type, "id": "300111207", "label": "Portuguese"},
	"japanese nationality": {"parent": Type, "id": "300018519", "label": "Japanese"},

	"style genre": {"parent": Type, "id":"300139140", "label": "Genre"},
	"style landscape": {"parent": Type, "id":"300015636", "label": "Landscape"},
	"style portrait": {"parent": Type, "id":"300015637", "label": "Portrait"},
	"style still life": {"parent": Type, "id":"300015638", "label": "Still Life"},

	"us dollars": {"parent": Currency, "id":"300411994", "label": "US Dollars"},
	"gb pounds": {"parent": Currency, "id":"300411998", "label": "British Pounds"},
	"dutch guilder": {"parent": Currency, "id":"300412019", "label": "Dutch Guilder"},
	"fr assignats": {"parent": Currency, "id":"300412157", "label": "French Assignats"},
	"at shillings": {"parent": Currency, "id":"300412158", "label": "Austrian Shillings"},
	"fr ecus": {"parent": Currency, "id":"300412159", "label": "French Ecus"},
	"de florins": {"parent": Currency, "id":"300412160", "label": "German Florins"},
	"gb guineas": {"parent": Currency, "id":"300412163", "label": "British Guineas"},
	"dk kroner": {"parent": Currency, "id":"300412164", "label": "Danish Kroner"},
	"fr livres": {"parent": Currency, "id":"300412165", "label": "French Livres"},
	"fr louis": {"parent": Currency, "id":"300412166", "label": "French Louis coins"},
	"de reichsmarks": {"parent": Currency, "id":"300412169", "label": "German Reichsmarks"},
	"reichsthalers": {"parent": Currency, "id":"300412170", "label": "Reichsthalers"},
	"ch francs": {"parent": Currency, "id":"300412001", "label": "Swiss Francs"},
	"fr francs": {"parent": Currency, "id":"300412016", "label": "French Francs"},
	"it lira": {"parent": Currency, "id":"300412015", "label": "Italian Lira"}
}

for (name,v) in identity_instances.items():
	register_instance(name, v['parent'], v['id'], v['label'])

# pen, pencil, card, cardboard, porcelain, wax, ceramic, plaster
# crayon, millboard, gouache, brass, stone, lead, iron, clay,
# alabaster, limestone

aat_culture_mapping = {
	"italian": "300111198",
	"german": "300111192",
	"dutch": "300020929"
}

def add_art_setter():
	# Linked.Art profile requires aat:300133025 on all artworks
	# Art can be a ManMadeObject or an InformationObject
	# set it by adding art=1 to the constructor

	def art_post_init(self, **kw):
		super(ManMadeObject, self)._post_init(**kw)
		if "art" in kw:
			self.classified_as = instances['artwork']
	ManMadeObject._post_init = art_post_init

	def art2_post_init(self, **kw):
		if "art" in kw:
			self.classified_as = instances['artwork']
		super(InformationObject, self)._post_init(**kw)
	InformationObject._post_init = art2_post_init

def add_proxy_wrapper():
	# There isn't an inverse of proxyFor / proxyIn.
	# So we need to use a JSON-LD @reverse property `proxies`
	# This adds it, as it's not part of the ontology proper.

	Aggregation.proxies = []
	def aggr_post_init(self, **kw):
		self.proxies = []
	Aggregation._post_init = aggr_post_init

	def proxy_set_magic_resource(self, which, value, inversed=False):
		super(Proxy, self)._set_magic_resource(which, value, inversed)
		# Look up what the context maps ore:proxyIn to
		pin = factory.context_rev.get('ore:proxyIn', 'proxyIn')
		if which == 'proxyIn':
			value.proxies.append(self)
	Proxy._set_magic_resource = proxy_set_magic_resource

def add_attribute_assignment_check():
	# Allow references to properties in p2 on AttrAssign
	# Validate that the property is allowed in assigned
	# either on set, or when assigned is set
		
	p2 = factory.context_rev.get('crm:P2_has_type', 'classified_as')
	ass = factory.context_rev.get('crm:P141_assigned', 'assigned')
	assto = factory.context_rev.get('crm:P140:assigned_attribute_to', 'assigned_to')
	phase_rel = factory.context_rev.get('la:relationship', 'relationship')
	phase_of = factory.context_rev.get('la:phase_of', 'phase_of')
	phase_entity = factory.context_rev.get('la:related_entity', 'related_entity')

	def aa_set_assigned(self, value):
		assto_res = getattr(self, assto, None)
		if assto_res:
			p2_res = getattr(self, p2, None)
			assto_res._check_prop(p2_res, value)
		object.__setattr__(self, ass, value)
	setattr(AttributeAssignment, "set_%s" % ass, aa_set_assigned)

	def aa_set_assigned_to(self, value):
		ass_res = getattr(self, ass, None)
		p2_res = getattr(self, p2, None)		
		if ass_res and p2_res:
			# unmap the URI to property name
			value._check_prop(p2_res, ass_res)
		object.__setattr__(self, assto, value)
	setattr(AttributeAssignment, "set_%s" % assto, aa_set_assigned_to)

	def aa_set_classified_as(self, value):
		ass_res = getattr(self, ass, None)
		assto_res = getattr(self, assto, None)
		if not assto_res:
			# override
			assto_res = BaseResource()
		if ass_res:
			assto_res._check_prop(value, ass_res)
		object.__setattr__(self, p2, value)
	setattr(AttributeAssignment, "set_%s" % p2, aa_set_classified_as)

	def phase_set_relationship(self, value):
		# XXX do same checking as above
		object.__setattr__(self, phase_rel, value)
	setattr(Phase, "set_%s" % phase_rel, phase_set_relationship)		





from lxml import etree
import codecs
import json
import sys

PROFILE_ONLY = '--profile' in sys.argv
default_key_order = 10000

NS = {'rdf':"http://www.w3.org/1999/02/22-rdf-syntax-ns#",
	'xsd':"http://www.w3.org/2001/XMLSchema#",
	'rdfs':"http://www.w3.org/2000/01/rdf-schema#",
	'dcterms':"http://purl.org/dc/terms/",
	'owl':"http://www.w3.org/2002/07/owl#",
	'crm':"http://www.cidoc-crm.org/cidoc-crm/",
	'xml': "http://www.w3.org/XML/1998/namespace",
	'ore': "http://www.openarchives.org/ore/terms/",
	'la': "https://linked.art/ns/terms/",
	"skos": "http://www.w3.org/2004/02/skos/core#",
	"schema": "http://schema.org/",
	"dc": "http://purl.org/dc/elements/1.1/",
	"geo": "http://www.ics.forth.gr/isl/CRMgeo/",
	"sci": "http://www.ics.forth.gr/isl/CRMsci/"
}

# Order imposed by the library
# @context = 0, id = 1, rdf:type = 2
# rdfs:label = 5, rdf:value = 6, dc:description = 7

fh = file('../cromulent/data/key_order.json')
data = fh.read()
fh.close()
key_order_hash = json.loads(data)

# Allow configuration of overrides for the mapping of ontology to python/json
fh = file('../cromulent/data/overrides.json')
data = fh.read()
fh.close()
property_overrides = json.loads(data)

# Allow subsetting of CRM into in-use / not-in-use to enable the library
# to warn on instantiation of not-in-use properties or classes
fh = file('../cromulent/data/crm-profile.json')
data = fh.read()
fh.close()
profile_flags = json.loads(data)

fh = file('data/inverses.xml')
data = fh.read()
fh.close()
invdom = etree.XML(data)

stuff = []
propXHash = {}
classXHash = {}

def process_classes(dom):
	classes = dom.xpath("//rdfs:Class", namespaces=NS)
	for c in classes:
		name = c.xpath('@rdf:about', namespaces=NS)[0]
		for (pref,ns) in NS.items():
			if name.startswith(ns):
				name = name.replace(ns, "%s:" % pref)
				break

		useflag = str(profile_flags.get(name, 0))
		if classXHash.has_key(name):
			classXHash[name][0] = c
		else:
			classXHash[name] = [c, useflag]

		label = c.xpath('./rdfs:label[@xml:lang="en"]/text()', namespaces=NS)[0]
		try:
			comment = c.xpath('./rdfs:comment/text()', namespaces=NS)[0]
			comment = comment.strip()
			comment = comment.replace('\n', '\\n').replace('\t', ' ')
		except:
			comment = ""

		subClsL = c.xpath('./rdfs:subClassOf/@rdf:resource', namespaces=NS)
		if subClsL:
			# could be multiples
			subCls = '|'.join(subClsL)
			for s in subClsL:
				try:
					classXHash[s][1] = 3
				except KeyError:
					classXHash[s] = [None, 3]
		else:
			subCls = ""

		# Hack SP4 and 5 to be readable :(
		if name == "geo:SP4_Spatial_Coordinate_Reference_System":
			ccname = "CoordinateSystem"
		elif name == "geo:SP5_Geometric_Place_Expression":
			ccname = "Geometry"
		elif name == "geo:SP6_Declarative_Place":
			ccname = "DeclarativePlace"
		else:
			# Assume that we've done our job okay and put in overrides for NSS
			cidx = name.find(":")
			if cidx > -1:
				ccname = name[cidx+1:]			
			else:
				uc1 = name.find("_")
				ccname = name[uc1+1:]
				ccname = ccname.replace("_or_", "_Or_").replace("_of_", "_Of_")
				ccname = ccname.replace('-', '').replace('_', '')

		stuff.append([name, "class", ccname, label, comment, subCls, useflag])

def process_props(dom):
	props = dom.xpath("//rdf:Property",namespaces=NS)
	for p in props:
		name = p.xpath('@rdf:about', namespaces=NS)[0]

		for (pref,ns) in NS.items():
			if name.startswith(ns):
				name = name.replace(ns, "%s:" % pref)
				break		

		useflags = profile_flags.get(name, [0,0]) or [0,0]
		propXHash[name] = [p, useflags[0]]

		label = p.xpath('./rdfs:label[@xml:lang="en"]/text()', namespaces=NS)[0]
		try:
			comment = p.xpath('./rdfs:comment/text()', namespaces=NS)[0]
			comment = comment.strip()
			comment = comment.replace('\n', '\\n').replace('\t', ' ')
		except:
			comment = ""

		domn = p.xpath('./rdfs:domain/@rdf:resource', namespaces=NS)
		if domn:		
			domn = domn[0]
			for (k,v) in NS.items():
				domn = domn.replace(v,"%s:" % k)
		else:
			domn = ""
		rang = p.xpath('./rdfs:range/@rdf:resource', namespaces=NS)
		if rang:
			rang = rang[0]
			for (k,v) in NS.items():
				rang = rang.replace(v,"%s:" % k)
		else:
			rang = ""

		subProp = p.xpath('./rdfs:subPropertyOf/@rdf:resource', namespaces=NS)
		if subProp:
			subProp = subProp[0]
		else:
			subProp = ""

		inverse = dom.xpath('./owl:inverseOf/@rdf:resource', namespaces=NS)
		if not inverse:
			inverse = invdom.xpath('//rdf:Property[@rdf:about="%s"]/owl:inverseOf/@rdf:resource' % name, namespaces=NS)
		if inverse:
			inverse = inverse[0]
		else:
			inverse = ""

		cidx = name.find(":")
		if property_overrides.has_key(name):
			ccname = property_overrides[name]
		elif cidx > -1:
			ccname = name[cidx+1:]
		else:
			uc1 = name.find("_")
			pno = name[:uc1]
			if property_overrides.has_key(pno):
				ccname = property_overrides[pno]
			else:
				ccname = name[uc1+1:]
				ccname = ccname.replace("-", "")
				if ccname.startswith("is_"):
					ccname = ccname[3:]
				elif ccname.startswith("has_") or ccname.startswith("had_") or ccname.startswith("was_"):
					ccname = ccname[4:]

		koi = str(key_order_hash.get(ccname, default_key_order))

		# either 0, or [1/2, 0/1] for okay/warn, single/multiple
		stuff.append([name, "property", ccname, label, comment, subProp, domn, rang, inverse, koi, 
			str(useflags[0]), str(useflags[1])])


files = ['cidoc.xml', 'linkedart.xml', 'linkedart_crm_enhancements.xml']

for fn in files:
	print "processing: %s" % fn
	fh = file('data/%s' % fn)
	data = fh.read()
	fh.close()
	dom = etree.XML(data)
	process_classes(dom)
	process_props(dom)


# outdata = '\n'.join(['\t'.join(x) for x in stuff])
fh = codecs.open('../cromulent/data/crm_vocab.tsv', 'w', 'utf-8')
for l in stuff:
	name = l[0]
	line = '\t'.join(l) + "\n"	
	if classXHash.has_key(name):
		okay = classXHash[name][1]
	elif propXHash.has_key(name):
		okay = propXHash[name][1]
	else:
		okay = 0
		print "Could not find %s" % name
	if not PROFILE_ONLY or okay:
		fh.write(line)
fh.close()
